"""
控制台监控器
捕获/过滤控制台日志，自动检测 JS 错误、警告
"""
import asyncio
import re
from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional, List, Dict, Callable, Literal
from playwright.async_api import Page, ConsoleMessage
from loguru import logger


LogLevel = Literal["log", "warn", "error", "info", "debug"]


@dataclass
class ConsoleLogEntry:
    """控制台日志条目"""
    level: str
    message: str
    timestamp: datetime = field(default_factory=datetime.now)
    url: Optional[str] = None
    line_number: Optional[int] = None
    column_number: Optional[int] = None
    stack_trace: Optional[str] = None


@dataclass
class ConsoleFilterConfig:
    """日志过滤配置"""
    include: Optional[List[re.Pattern]] = None
    exclude: Optional[List[re.Pattern]] = None


@dataclass
class ConsoleMonitorConfig:
    """控制台监控配置"""
    levels: List[str] = field(default_factory=lambda: ["log", "warn", "error", "info", "debug"])
    capture_stack_trace: bool = True
    filters: Optional[ConsoleFilterConfig] = None


@dataclass
class ConsoleMonitorResult:
    """控制台监控结果"""
    total_count: int = 0
    by_level: Dict[str, int] = field(default_factory=dict)
    errors: List[ConsoleLogEntry] = field(default_factory=list)
    warnings: List[ConsoleLogEntry] = field(default_factory=list)
    logs: List[ConsoleLogEntry] = field(default_factory=list)


class ConsoleMonitor:
    """
    控制台监控器
    捕获浏览器控制台日志
    """

    def __init__(
        self,
        page: Page,
        config: Optional[ConsoleMonitorConfig] = None
    ):
        self.page = page
        self.config = config or ConsoleMonitorConfig()
        self._logs: List[ConsoleLogEntry] = []
        self._is_monitoring = False
        self._handlers: List[tuple] = []

    def start(self) -> None:
        """开始监控"""
        if self._is_monitoring:
            return

        self._logs = []
        self._is_monitoring = True

        # 监听控制台消息
        async def console_handler(msg: ConsoleMessage):
            await self._handle_console_message(msg)

        self.page.on("console", lambda msg: asyncio.create_task(console_handler(msg)))
        self._handlers.append(("console", console_handler))

        # 监听页面错误
        async def error_handler(error: Exception):
            self._add_log(ConsoleLogEntry(
                level="error",
                message=str(error),
                stack_trace=getattr(error, '__traceback__', None),
            ))

        self.page.on("pageerror", lambda e: asyncio.create_task(error_handler(e)))
        self._handlers.append(("pageerror", error_handler))

        logger.debug("控制台监控已启动")

    def stop(self) -> None:
        """停止监控"""
        if not self._is_monitoring:
            return

        self._is_monitoring = False
        self._handlers.clear()
        logger.debug("控制台监控已停止")

    async def _handle_console_message(self, msg: ConsoleMessage) -> None:
        """处理控制台消息"""
        level = msg.type

        # 过滤级别
        if level not in self.config.levels:
            return

        message = msg.text

        # 应用过滤规则
        if self.config.filters:
            include = self.config.filters.include
            exclude = self.config.filters.exclude

            if exclude and any(p.search(message) for p in exclude):
                return
            if include and not any(p.search(message) for p in include):
                return

        entry = ConsoleLogEntry(
            level=level,
            message=message,
        )

        # 获取位置信息
        try:
            location = msg.location
            if location:
                entry.url = location.get("url")
                entry.line_number = location.get("lineNumber")
                entry.column_number = location.get("columnNumber")
        except Exception:
            pass

        self._add_log(entry)

    def _add_log(self, entry: ConsoleLogEntry) -> None:
        """添加日志条目"""
        self._logs.append(entry)

    # ==================== 获取日志 ====================

    def get_logs(self) -> List[ConsoleLogEntry]:
        """获取所有日志"""
        return list(self._logs)

    def get_errors(self) -> List[ConsoleLogEntry]:
        """获取错误日志"""
        return [log for log in self._logs if log.level == "error"]

    def get_warnings(self) -> List[ConsoleLogEntry]:
        """获取警告日志"""
        return [log for log in self._logs if log.level == "warn"]

    def get_logs_by_level(self, level: str) -> List[ConsoleLogEntry]:
        """获取指定级别的日志"""
        return [log for log in self._logs if log.level == level]

    def get_result(self) -> ConsoleMonitorResult:
        """获取结果统计"""
        by_level: Dict[str, int] = {
            "log": 0,
            "warn": 0,
            "error": 0,
            "info": 0,
            "debug": 0,
        }

        for log in self._logs:
            if log.level in by_level:
                by_level[log.level] += 1

        return ConsoleMonitorResult(
            total_count=len(self._logs),
            by_level=by_level,
            errors=self.get_errors(),
            warnings=self.get_warnings(),
            logs=self.get_logs(),
        )

    def has_errors(self) -> bool:
        """检查是否有错误"""
        return any(log.level == "error" for log in self._logs)

    def has_warnings(self) -> bool:
        """检查是否有警告"""
        return any(log.level == "warn" for log in self._logs)

    def get_error_count(self) -> int:
        """获取错误数量"""
        return len(self.get_errors())

    def get_warning_count(self) -> int:
        """获取警告数量"""
        return len(self.get_warnings())

    # ==================== 日志操作 ====================

    def clear(self) -> None:
        """清空日志"""
        self._logs = []

    def export_json(self) -> str:
        """导出日志为 JSON"""
        import json
        return json.dumps([
            {
                "level": log.level,
                "message": log.message,
                "timestamp": log.timestamp.isoformat(),
                "url": log.url,
                "line": log.line_number,
                "column": log.column_number,
            }
            for log in self._logs
        ], ensure_ascii=False, indent=2)

    def export_text(self) -> str:
        """导出日志为文本"""
        lines = []
        for log in self._logs:
            timestamp = log.timestamp.isoformat()
            location = ""
            if log.url:
                location = f" ({log.url}:{log.line_number}:{log.column_number})"
            lines.append(f"[{timestamp}] [{log.level.upper()}]{location} {log.message}")
        return "\n".join(lines)

    def print_summary(self) -> None:
        """打印日志摘要"""
        result = self.get_result()

        print("\n=== 控制台监控摘要 ===")
        print(f"总日志数: {result.total_count}")
        print(f"  - 错误: {result.by_level.get('error', 0)}")
        print(f"  - 警告: {result.by_level.get('warn', 0)}")
        print(f"  - 信息: {result.by_level.get('info', 0)}")
        print(f"  - 调试: {result.by_level.get('debug', 0)}")
        print(f"  - 日志: {result.by_level.get('log', 0)}")

        if result.errors:
            print("\n错误详情:")
            for i, err in enumerate(result.errors, 1):
                print(f"  {i}. {err.message}")
                if err.url:
                    print(f"     位置: {err.url}:{err.line_number}")

    # ==================== 等待方法 ====================

    async def wait_for_log(
        self,
        pattern: str,
        timeout: float = 30000,
        level: Optional[str] = None,
    ) -> Optional[ConsoleLogEntry]:
        """
        等待特定日志

        Args:
            pattern: 日志匹配模式
            timeout: 超时时间（毫秒）
            level: 日志级别

        Returns:
            Optional[ConsoleLogEntry]: 匹配的日志条目
        """
        regex = re.compile(pattern) if isinstance(pattern, str) else pattern
        start_time = asyncio.get_event_loop().time()

        while (asyncio.get_event_loop().time() - start_time) * 1000 < timeout:
            for log in self._logs:
                if level and log.level != level:
                    continue
                if regex.search(log.message):
                    return log

            await asyncio.sleep(0.1)

        return None

    async def wait_for_error(self, timeout: float = 30000) -> Optional[ConsoleLogEntry]:
        """等待错误出现"""
        return await self.wait_for_log(r".*", timeout=timeout, level="error")

    # ==================== 断言方法 ====================

    def assert_no_errors(self) -> None:
        """断言无错误"""
        errors = self.get_errors()
        if errors:
            messages = "\n  ".join(e.message for e in errors)
            raise AssertionError(f"控制台存在错误:\n  {messages}")

    def assert_no_warnings(self) -> None:
        """断言无警告"""
        warnings = self.get_warnings()
        if warnings:
            messages = "\n  ".join(w.message for w in warnings)
            raise AssertionError(f"控制台存在警告:\n  {messages}")

    def assert_contains(self, pattern: str, level: Optional[str] = None) -> None:
        """断言包含日志"""
        regex = re.compile(pattern)
        logs = self.get_logs_by_level(level) if level else self._logs

        if not any(regex.search(log.message) for log in logs):
            raise AssertionError(f"未找到匹配的日志: {pattern}")


def create_console_monitor(
    page: Page,
    config: Optional[ConsoleMonitorConfig] = None
) -> ConsoleMonitor:
    """创建控制台监控器实例"""
    return ConsoleMonitor(page, config)



"""
分页导航器
智能分页导航，自动识别下一页按钮
"""
import asyncio
from dataclasses import dataclass, field
from typing import Optional, Callable, List
from playwright.async_api import Page, Locator
from loguru import logger


@dataclass
class PaginationConfig:
    """分页配置"""
    next_selector: str = 'a.next, button.next, [aria-label="下一页"], [aria-label="Next"]'
    page_selector: Optional[str] = None  # 页码选择器
    max_pages: int = 100
    wait_time: float = 0.5  # 秒
    stop_condition: Optional[Callable[[], bool]] = None
    on_before_page: Optional[Callable[[int], None]] = None
    on_after_page: Optional[Callable[[int], None]] = None


@dataclass
class PaginationResult:
    """分页结果"""
    total_pages: int = 0
    reached_end: bool = False
    stopped_by_condition: bool = False
    duration: float = 0.0  # 毫秒


class PaginationNavigator:
    """
    分页导航器
    自动处理分页导航
    """

    # 常用选择器列表
    NEXT_SELECTORS = [
        'a.next',
        'button.next',
        '[aria-label="下一页"]',
        '[aria-label="Next"]',
        '.pagination a:has-text("下一页")',
        '.pagination a:has-text(">")',
        '.pagination a:has-text("›")',
        '.pagination a:has-text("»")',
        'a[rel="next"]',
        'li.next a',
    ]

    def __init__(self, page: Page, config: Optional[PaginationConfig] = None):
        self.page = page
        self.config = config or PaginationConfig()
        self._current_page = 1
        self._visited_pages: set = set()

    async def navigate(self) -> PaginationResult:
        """
        执行分页导航

        Returns:
            PaginationResult: 导航结果
        """
        import time
        start_time = time.time()

        self._current_page = 1
        self._visited_pages.clear()
        total_pages = 1
        reached_end = False
        stopped_by_condition = False

        while self._current_page <= self.config.max_pages:
            # 记录当前页
            self._visited_pages.add(self._current_page)

            # 检查停止条件
            if self.config.stop_condition:
                should_stop = self.config.stop_condition()
                if asyncio.iscoroutinefunction(self.config.stop_condition):
                    should_stop = await should_stop
                if should_stop:
                    stopped_by_condition = True
                    break

            # 翻页前回调
            if self.config.on_before_page:
                if asyncio.iscoroutinefunction(self.config.on_before_page):
                    await self.config.on_before_page(self._current_page)
                else:
                    self.config.on_before_page(self._current_page)

            # 尝试翻到下一页
            has_next_page = await self.go_to_next_page()

            if not has_next_page:
                reached_end = True
                break

            total_pages += 1
            self._current_page += 1

            # 翻页后回调
            if self.config.on_after_page:
                if asyncio.iscoroutinefunction(self.config.on_after_page):
                    await self.config.on_after_page(self._current_page)
                else:
                    self.config.on_after_page(self._current_page)

            # 等待页面加载
            await self._wait_for_page_load()

        duration = (time.time() - start_time) * 1000

        logger.debug(
            f"分页导航完成: pages={total_pages}, "
            f"reached_end={reached_end}, duration={duration:.0f}ms"
        )

        return PaginationResult(
            total_pages=total_pages,
            reached_end=reached_end,
            stopped_by_condition=stopped_by_condition,
            duration=duration,
        )

    async def navigate_all(
        self,
        next_selector: str,
        max_pages: int = 100,
        on_page: Optional[Callable[[int], None]] = None,
    ) -> PaginationResult:
        """
        遍历所有分页

        Args:
            next_selector: 下一页按钮选择器
            max_pages: 最大页数
            on_page: 每页回调函数

        Returns:
            PaginationResult: 导航结果
        """
        config = PaginationConfig(
            next_selector=next_selector,
            max_pages=max_pages,
            on_after_page=on_page,
        )
        old_config = self.config
        self.config = config
        result = await self.navigate()
        self.config = old_config
        return result

    async def extract_all_pages(
        self,
        item_selector: str,
        next_selector: str,
        max_pages: int = 100,
    ) -> list:
        """
        提取所有页面的数据

        Args:
            item_selector: 元素选择器
            next_selector: 下一页选择器
            max_pages: 最大页数

        Returns:
            list: 所有页面提取的数据
        """
        all_items = []

        async def extract_page(page_num: int):
            items = await self.page.locator(item_selector).all_text_contents()
            all_items.extend(items)

        config = PaginationConfig(
            next_selector=next_selector,
            max_pages=max_pages,
            on_after_page=extract_page,
        )
        old_config = self.config
        self.config = config
        await self.navigate()
        self.config = old_config

        return all_items

    async def go_to_next_page(self) -> bool:
        """翻到下一页"""
        next_button = self.page.locator(self.config.next_selector)

        # 检查下一页按钮是否存在且可点击
        try:
            is_visible = await next_button.is_visible()
        except Exception:
            return False

        if not is_visible:
            return False

        if await self._is_button_disabled(next_button):
            return False

        # 点击下一页
        await next_button.click()
        logger.debug(f"翻到下一页: {self._current_page + 1}")
        return True

    async def go_to_page(self, page_number: int) -> bool:
        """跳转到指定页"""
        if self.config.page_selector:
            page_button = self.page.locator(
                f'{self.config.page_selector}:has-text("{page_number}")'
            )

            try:
                is_visible = await page_button.is_visible()
            except Exception:
                return False

            if is_visible:
                await page_button.click()
                self._current_page = page_number
                await self._wait_for_page_load()
                return True

        return False

    async def go_to_first_page(self) -> bool:
        """跳转到第一页"""
        first_button = self.page.locator(
            'a.first, button.first, [aria-label="第一页"], [aria-label="First"]'
        )

        try:
            is_visible = await first_button.is_visible()
        except Exception:
            return False

        if is_visible:
            await first_button.click()
            self._current_page = 1
            await self._wait_for_page_load()
            return True

        return False

    async def go_to_last_page(self) -> bool:
        """跳转到最后一页"""
        last_button = self.page.locator(
            'a.last, button.last, [aria-label="最后一页"], [aria-label="Last"]'
        )

        try:
            is_visible = await last_button.is_visible()
        except Exception:
            return False

        if is_visible:
            await last_button.click()
            await self._wait_for_page_load()
            return True

        return False

    def get_current_page(self) -> int:
        """获取当前页码"""
        return self._current_page

    def get_visited_pages(self) -> List[int]:
        """获取已访问的页数"""
        return sorted(list(self._visited_pages))

    async def has_next_page(self) -> bool:
        """检测是否有下一页"""
        next_button = self.page.locator(self.config.next_selector)
        try:
            is_visible = await next_button.is_visible()
        except Exception:
            return False

        if not is_visible:
            return False
        return not await self._is_button_disabled(next_button)

    async def has_previous_page(self) -> bool:
        """检测是否有上一页"""
        prev_button = self.page.locator(
            'a.prev, button.prev, [aria-label="上一页"], [aria-label="Previous"]'
        )
        try:
            is_visible = await prev_button.is_visible()
        except Exception:
            return False

        if not is_visible:
            return False
        return not await self._is_button_disabled(prev_button)

    async def get_total_pages(self) -> Optional[int]:
        """获取总页数（如果页面显示）"""
        try:
            total_text = await self.page.locator(
                '.pagination-total, .page-total, [class*="total"]'
            ).text_content()

            if total_text:
                import re
                match = re.search(r'(\d+)', total_text)
                if match:
                    return int(match.group(1))
        except Exception:
            pass

        return None

    async def _is_button_disabled(self, button: Locator) -> bool:
        """检测按钮是否禁用"""
        try:
            class_name = await button.get_attribute("class") or ""
            disabled = await button.get_attribute("disabled")
            aria_disabled = await button.get_attribute("aria-disabled")

            return (
                disabled is not None or
                aria_disabled == "true" or
                "disabled" in class_name
            )
        except Exception:
            return True

    async def _wait_for_page_load(self):
        """等待页面加载"""
        await asyncio.sleep(self.config.wait_time)
        try:
            await self.page.wait_for_load_state("domcontentloaded", timeout=5000)
        except Exception:
            pass

    @classmethod
    async def detect_pagination_selector(cls, page: Page) -> Optional[str]:
        """
        自动检测分页选择器

        Args:
            page: 页面对象

        Returns:
            Optional[str]: 检测到的选择器，未找到返回 None
        """
        for selector in cls.NEXT_SELECTORS:
            try:
                element = page.locator(selector)
                is_visible = await element.is_visible()
                if is_visible:
                    return selector
            except Exception:
                continue

        return None


def create_pagination_navigator(
    page: Page,
    config: Optional[PaginationConfig] = None
) -> PaginationNavigator:
    """创建分页导航器实例"""
    return PaginationNavigator(page, config)



"""
测试辅助类
整合 test_helpers 子模块，提供统一访问入口
"""
from typing import Callable, Optional
from playwright.async_api import Page

from ..test_helpers import (
    AccessibilityChecker,
    AccessibilityCheckConfig,
    UserFactory,
    ProductFactory,
    OrderFactory,
    FactoryConfig,
    MockServer,
    MockConfig,
)


class TestHelper:
    """
    测试辅助类
    整合 test_helpers 子模块

    使用方式:
        browser.test_helpers.accessibility  # AccessibilityChecker
        browser.test_helpers.user_factory   # UserFactory
        browser.test_helpers.product_factory # ProductFactory
        browser.test_helpers.mock           # MockServer
    """

    def __init__(self, get_page: Callable[[], Page]):
        """
        初始化测试辅助类

        Args:
            get_page: 获取当前 Page 对象的回调函数
        """
        self._get_page = get_page
        self._accessibility: Optional[AccessibilityChecker] = None
        self._accessibility_config: Optional[AccessibilityCheckConfig] = None
        self._user_factory: Optional[UserFactory] = None
        self._product_factory: Optional[ProductFactory] = None
        self._order_factory: Optional[OrderFactory] = None
        self._mock: Optional[MockServer] = None
        self._factory_config: Optional[FactoryConfig] = None
        self._mock_config: Optional[MockConfig] = None

    @property
    def accessibility(self) -> AccessibilityChecker:
        """
        获取可访问性检查器

        Returns:
            AccessibilityChecker: 可访问性检查器实例
        """
        if self._accessibility is None:
            self._accessibility = AccessibilityChecker(
                self._get_page(),
                self._accessibility_config
            )
        return self._accessibility

    @property
    def user_factory(self) -> UserFactory:
        """
        获取用户数据工厂

        Returns:
            UserFactory: 用户数据工厂实例
        """
        if self._user_factory is None:
            self._user_factory = UserFactory(self._factory_config)
        return self._user_factory

    @property
    def product_factory(self) -> ProductFactory:
        """
        获取产品数据工厂

        Returns:
            ProductFactory: 产品数据工厂实例
        """
        if self._product_factory is None:
            self._product_factory = ProductFactory(self._factory_config)
        return self._product_factory

    @property
    def order_factory(self) -> OrderFactory:
        """
        获取订单数据工厂

        Returns:
            OrderFactory: 订单数据工厂实例
        """
        if self._order_factory is None:
            self._order_factory = OrderFactory(self._factory_config)
        return self._order_factory

    @property
    def mock(self) -> MockServer:
        """
        获取 Mock 服务器

        Returns:
            MockServer: Mock 服务器实例
        """
        if self._mock is None:
            self._mock = MockServer(self._get_page(), self._mock_config)
        return self._mock

    def set_accessibility_config(self, config: AccessibilityCheckConfig):
        """
        设置可访问性检查配置

        Args:
            config: 可访问性检查配置
        """
        self._accessibility_config = config
        self._accessibility = None

    def set_factory_config(self, config: FactoryConfig):
        """
        设置数据工厂配置

        Args:
            config: 数据工厂配置
        """
        self._factory_config = config
        self._user_factory = None
        self._product_factory = None
        self._order_factory = None

    def set_mock_config(self, config: MockConfig):
        """
        设置 Mock 服务器配置

        Args:
            config: Mock 服务器配置
        """
        self._mock_config = config
        self._mock = None

    def reset(self):
        """
        重置所有实例
        当切换标签页时调用，确保使用正确的 Page 对象
        """
        self._accessibility = None
        self._mock = None



"""
任务队列管理器
支持优先级和并发控制
"""
import asyncio
import uuid
from typing import Optional, Dict, Any, Callable, Awaitable, TypeVar, List
from datetime import datetime
from loguru import logger

from .types import TaskStatus, TaskQueueConfig, QueueTask

T = TypeVar('T')


class TaskQueue:
    """任务队列管理器，支持优先级和并发控制"""

    def __init__(self, config: Optional[TaskQueueConfig] = None):
        self.config = config or TaskQueueConfig()
        self._queue: asyncio.PriorityQueue = asyncio.PriorityQueue()
        self._tasks: Dict[str, QueueTask] = {}
        self._running_tasks: Dict[str, asyncio.Task] = {}
        self._results: Dict[str, Any] = {}
        self._is_running = False
        self._workers: List[asyncio.Task] = []
        self._semaphore: Optional[asyncio.Semaphore] = None

    async def start(self):
        if self._is_running:
            return
        self._is_running = True
        self._semaphore = asyncio.Semaphore(self.config.max_concurrent)
        for i in range(self.config.max_concurrent):
            worker = asyncio.create_task(self._worker(i))
            self._workers.append(worker)

    async def stop(self, wait: bool = True):
        self._is_running = False
        if wait:
            while self._running_tasks:
                await asyncio.sleep(0.1)
        for worker in self._workers:
            worker.cancel()
        self._workers.clear()

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.stop()

    def submit(
        self,
        handler: Callable[..., Awaitable[T]],
        name: Optional[str] = None,
        priority: int = 5,
        args: Optional[tuple] = None,
        kwargs: Optional[Dict] = None,
        dependencies: Optional[List[str]] = None,
        max_retries: int = 3
    ) -> str:
        task_id = str(uuid.uuid4())[:8]
        task = QueueTask(
            id=task_id, name=name or handler.__name__, handler=handler,
            priority=priority, args=args or (), kwargs=kwargs or {},
            dependencies=dependencies or [], max_retries=max_retries,
        )
        self._tasks[task_id] = task
        self._queue.put_nowait((-priority, task.created_at.timestamp(), task_id))
        return task_id

    async def wait_for(self, task_id: str, timeout: Optional[float] = None) -> Any:
        timeout = timeout or self.config.timeout
        start_time = datetime.now()
        while True:
            task = self._tasks.get(task_id)
            if not task:
                raise ValueError(f"任务不存在: {task_id}")
            if task.status == TaskStatus.COMPLETED:
                return self._results.get(task_id)
            if task.status == TaskStatus.FAILED:
                raise RuntimeError(f"任务失败: {task.error}")
            if task.status == TaskStatus.CANCELLED:
                raise asyncio.CancelledError("任务已取消")
            if (datetime.now() - start_time).total_seconds() > timeout:
                raise asyncio.TimeoutError("等待任务超时")
            await asyncio.sleep(0.1)

    def get_task(self, task_id: str) -> Optional[QueueTask]:
        return self._tasks.get(task_id)

    def get_stats(self) -> Dict[str, Any]:
        stats = {"total": len(self._tasks)}
        for status in TaskStatus:
            stats[status.value] = sum(1 for t in self._tasks.values() if t.status == status)
        return stats

    async def _worker(self, worker_id: int):
        while self._is_running:
            try:
                try:
                    _, _, task_id = await asyncio.wait_for(self._queue.get(), timeout=1.0)
                except asyncio.TimeoutError:
                    continue
                task = self._tasks.get(task_id)
                if not task or task.status == TaskStatus.CANCELLED:
                    continue
                async with self._semaphore:
                    await self._execute_task(task)
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error(f"工作协程 {worker_id} 错误: {e}")

    async def _execute_task(self, task: QueueTask):
        task.status = TaskStatus.RUNNING
        task.started_at = datetime.now()
        try:
            result = await asyncio.wait_for(
                task.handler(*task.args, **task.kwargs),
                timeout=self.config.timeout
            )
            task.result = result
            task.status = TaskStatus.COMPLETED
            self._results[task.id] = result
        except asyncio.TimeoutError:
            task.error = "任务超时"
            task.status = TaskStatus.FAILED
        except Exception as e:
            task.error = str(e)
            if task.retry_count < task.max_retries:
                task.retry_count += 1
                task.status = TaskStatus.PENDING
                await asyncio.sleep(self.config.retry_delay)
                self._queue.put_nowait((-task.priority, task.created_at.timestamp(), task.id))
            else:
                task.status = TaskStatus.FAILED
        finally:
            task.completed_at = datetime.now()

    def cancel(self, task_id: str) -> bool:
        """取消任务"""
        task = self._tasks.get(task_id)
        if task and task.status == TaskStatus.PENDING:
            task.status = TaskStatus.CANCELLED
            return True
        return False

    def get_pending_count(self) -> int:
        """获取待处理任务数"""
        return sum(1 for t in self._tasks.values() if t.status == TaskStatus.PENDING)

    def get_running_count(self) -> int:
        """获取运行中任务数"""
        return sum(1 for t in self._tasks.values() if t.status == TaskStatus.RUNNING)



"""
浏览器模块类型定义
"""
from dataclasses import dataclass
from playwright.async_api import Page


@dataclass
class Tab:
    """标签页"""
    id: str
    page: Page
    name: str = ""
    url: str = ""

    def __post_init__(self):
        self.url = self.page.url



"""
浏览器上下文管理
高级上下文管理功能
"""
import asyncio
from typing import Optional, Dict, Any, List
from playwright.async_api import BrowserContext, Page, Browser
from loguru import logger
from dataclasses import dataclass
from pathlib import Path


@dataclass
class ContextProfile:
    """上下文配置"""
    name: str
    storage_state: Optional[str] = None
    viewport: Optional[Dict[str, int]] = None
    user_agent: Optional[str] = None
    locale: Optional[str] = None
    timezone: Optional[str] = None
    geolocation: Optional[Dict[str, float]] = None
    permissions: Optional[List[str]] = None


class ContextManager:
    """
    浏览器上下文管理器
    管理多个浏览器上下文
    """
    
    def __init__(self, browser: Browser):
        self.browser = browser
        self._contexts: Dict[str, BrowserContext] = {}
        self._pages: Dict[str, Page] = {}
    
    async def create_context(
        self,
        name: str,
        profile: ContextProfile = None,
        **kwargs,
    ) -> BrowserContext:
        """
        创建浏览器上下文
        
        Args:
            name: 上下文名称
            profile: 上下文配置
            **kwargs: 额外参数
            
        Returns:
            BrowserContext: 浏览器上下文
        """
        if name in self._contexts:
            raise ValueError(f"上下文已存在: {name}")
        
        options = {}
        
        if profile:
            if profile.storage_state:
                options["storage_state"] = profile.storage_state
            if profile.viewport:
                options["viewport"] = profile.viewport
            if profile.user_agent:
                options["user_agent"] = profile.user_agent
            if profile.locale:
                options["locale"] = profile.locale
            if profile.timezone:
                options["timezone_id"] = profile.timezone
            if profile.geolocation:
                options["geolocation"] = profile.geolocation
            if profile.permissions:
                options["permissions"] = profile.permissions
        
        options.update(kwargs)
        
        context = await self.browser.new_context(**options)
        self._contexts[name] = context
        
        logger.info(f"创建上下文: {name}")
        return context
    
    async def get_context(self, name: str) -> Optional[BrowserContext]:
        """获取上下文"""
        return self._contexts.get(name)
    
    async def create_page(self, context_name: str, page_name: str) -> Page:
        """
        在上下文中创建页面
        
        Args:
            context_name: 上下文名称
            page_name: 页面名称
            
        Returns:
            Page: 页面对象
        """
        context = self._contexts.get(context_name)
        if not context:
            raise ValueError(f"上下文不存在: {context_name}")
        
        page = await context.new_page()
        self._pages[page_name] = page
        
        logger.info(f"创建页面: {page_name} (context: {context_name})")
        return page
    
    async def get_page(self, page_name: str) -> Optional[Page]:
        """获取页面"""
        return self._pages.get(page_name)
    
    async def close_context(self, name: str):
        """关闭上下文"""
        if name in self._contexts:
            await self._contexts[name].close()
            del self._contexts[name]
            logger.info(f"关闭上下文: {name}")
    
    async def close_all(self):
        """关闭所有上下文"""
        for name in list(self._contexts.keys()):
            await self.close_context(name)
    
    def list_contexts(self) -> List[str]:
        """列出所有上下文"""
        return list(self._contexts.keys())
    
    def list_pages(self) -> List[str]:
        """列出所有页面"""
        return list(self._pages.keys())
    
    async def save_state(self, context_name: str, filepath: str):
        """保存上下文状态"""
        context = self._contexts.get(context_name)
        if not context:
            raise ValueError(f"上下文不存在: {context_name}")
        
        await context.storage_state(path=filepath)
        logger.info(f"保存状态: {filepath}")
    
    async def load_state(self, context_name: str, filepath: str) -> BrowserContext:
        """加载上下文状态"""
        if context_name in self._contexts:
            raise ValueError(f"上下文已存在: {context_name}")
        
        context = await self.browser.new_context(storage_state=filepath)
        self._contexts[context_name] = context
        
        logger.info(f"加载状态: {filepath}")
        return context


class IsolatedSession:
    """
    隔离会话
    提供完全隔离的浏览器会话
    """
    
    def __init__(
        self,
        name: str,
        browser: Browser,
        isolated_storage: bool = True,
    ):
        self.name = name
        self.browser = browser
        self.isolated_storage = isolated_storage
        self._context: Optional[BrowserContext] = None
        self._page: Optional[Page] = None
        self._storage_path: Optional[str] = None
    
    async def start(self) -> Page:
        """启动会话"""
        options = {}
        
        if self.isolated_storage:
            self._storage_path = f"./sessions/{self.name}_state.json"
            Path(self._storage_path).parent.mkdir(parents=True, exist_ok=True)
            
            if Path(self._storage_path).exists():
                options["storage_state"] = self._storage_path
        
        self._context = await self.browser.new_context(**options)
        self._page = await self._context.new_page()
        
        logger.info(f"隔离会话已启动: {self.name}")
        return self._page
    
    async def save(self):
        """保存会话状态"""
        if self._context and self._storage_path:
            await self._context.storage_state(path=self._storage_path)
            logger.info(f"会话状态已保存: {self.name}")
    
    async def close(self):
        """关闭会话"""
        if self._context:
            await self.save()
            await self._context.close()
            self._context = None
            self._page = None
            logger.info(f"隔离会话已关闭: {self.name}")
    
    @property
    def page(self) -> Optional[Page]:
        """获取页面对象"""
        return self._page
    
    async def __aenter__(self) -> Page:
        return await self.start()
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()


class MultiUserSession:
    """
    多用户会话管理
    同时管理多个用户的浏览器会话
    """
    
    def __init__(self, browser: Browser):
        self.browser = browser
        self._sessions: Dict[str, IsolatedSession] = {}
    
    async def create_user(self, username: str) -> Page:
        """创建用户会话"""
        if username in self._sessions:
            raise ValueError(f"用户会话已存在: {username}")
        
        session = IsolatedSession(username, self.browser)
        self._sessions[username] = session
        
        return await session.start()
    
    async def get_user_page(self, username: str) -> Optional[Page]:
        """获取用户页面"""
        session = self._sessions.get(username)
        return session.page if session else None
    
    async def switch_user(self, username: str) -> Page:
        """切换用户"""
        page = await self.get_user_page(username)
        if not page:
            raise ValueError(f"用户会话不存在: {username}")
        return page
    
    async def close_user(self, username: str):
        """关闭用户会话"""
        session = self._sessions.get(username)
        if session:
            await session.close()
            del self._sessions[username]
    
    async def close_all(self):
        """关闭所有用户会话"""
        for username in list(self._sessions.keys()):
            await self.close_user(username)
    
    def list_users(self) -> List[str]:
        """列出所有用户"""
        return list(self._sessions.keys())
    
    async def broadcast(self, action):
        """向所有用户广播操作"""
        import asyncio
        
        tasks = []
        for session in self._sessions.values():
            if session.page:
                if asyncio.iscoroutinefunction(action):
                    tasks.append(action(session.page))
                else:
                    action(session.page)
        
        if tasks:
            await asyncio.gather(*tasks)


class SessionPool:
    """
    会话池
    复用浏览器会话
    """
    
    def __init__(
        self,
        browser: Browser,
        pool_size: int = 5,
    ):
        self.browser = browser
        self.pool_size = pool_size
        self._pool: List[BrowserContext] = []
        self._in_use: Dict[int, BrowserContext] = {}
        self._counter = 0
    
    async def initialize(self):
        """初始化会话池"""
        for _ in range(self.pool_size):
            context = await self.browser.new_context()
            self._pool.append(context)
        
        logger.info(f"会话池已初始化: {self.pool_size} 个会话")
    
    async def acquire(self) -> tuple:
        """
        获取会话
        
        Returns:
            tuple: (session_id, page)
        """
        if not self._pool:
            raise RuntimeError("会话池为空，请先初始化")
        
        context = self._pool.pop()
        page = await context.new_page()
        
        self._counter += 1
        session_id = self._counter
        self._in_use[session_id] = context
        
        logger.debug(f"获取会话: {session_id}")
        return session_id, page
    
    async def release(self, session_id: int):
        """释放会话"""
        if session_id not in self._in_use:
            return
        
        context = self._in_use.pop(session_id)
        
        # 关闭所有页面
        for page in context.pages:
            await page.close()
        
        self._pool.append(context)
        logger.debug(f"释放会话: {session_id}")
    
    async def close(self):
        """关闭会话池"""
        for context in self._pool:
            await context.close()
        
        for context in self._in_use.values():
            await context.close()
        
        self._pool.clear()
        self._in_use.clear()
        
        logger.info("会话池已关闭")



"""
定时任务调度器
支持定时执行自动化任务
"""
import asyncio
from typing import Optional, Callable, List, Dict, Any
from datetime import datetime, timedelta
from croniter import croniter
from loguru import logger
from dataclasses import dataclass, field
from enum import Enum


class TaskStatus(Enum):
    """任务状态"""
    PENDING = "pending"
    RUNNING = "running"
    COMPLETED = "completed"
    FAILED = "failed"
    CANCELLED = "cancelled"


@dataclass
class ScheduledTask:
    """定时任务"""
    id: str
    name: str
    cron: str  # cron表达式
    handler: Callable
    next_run: Optional[datetime] = None
    last_run: Optional[datetime] = None
    status: TaskStatus = TaskStatus.PENDING
    run_count: int = 0
    error: Optional[str] = None
    enabled: bool = True
    params: Dict[str, Any] = field(default_factory=dict)
    
    def calculate_next_run(self) -> datetime:
        """计算下次运行时间"""
        iter = croniter(self.cron, datetime.now())
        return iter.get_next(datetime)


class TaskScheduler:
    """
    任务调度器
    支持cron表达式的定时任务调度
    """
    
    def __init__(self):
        self._tasks: Dict[str, ScheduledTask] = {}
        self._running: bool = False
        self._task_id_counter: int = 0
    
    def add_task(
        self,
        name: str,
        cron: str,
        handler: Callable,
        params: Dict[str, Any] = None,
    ) -> str:
        """
        添加定时任务
        
        Args:
            name: 任务名称
            cron: cron表达式 (如 "0 9 * * *" 表示每天9点)
            handler: 任务处理函数
            params: 任务参数
            
        Returns:
            str: 任务ID
            
        Example:
            scheduler.add_task(
                "每天9点发简报",
                "0 9 * * *",
                send_briefing,
                {"recipient": "user@example.com"}
            )
        """
        self._task_id_counter += 1
        task_id = f"task_{self._task_id_counter}"
        
        task = ScheduledTask(
            id=task_id,
            name=name,
            cron=cron,
            handler=handler,
            params=params or {},
        )
        
        task.next_run = task.calculate_next_run()
        self._tasks[task_id] = task
        
        logger.info(f"添加任务: {name}, cron={cron}, 下次运行={task.next_run}")
        return task_id
    
    def remove_task(self, task_id: str):
        """移除任务"""
        if task_id in self._tasks:
            del self._tasks[task_id]
            logger.info(f"移除任务: {task_id}")
    
    def enable_task(self, task_id: str):
        """启用任务"""
        if task_id in self._tasks:
            self._tasks[task_id].enabled = True
            logger.info(f"启用任务: {task_id}")
    
    def disable_task(self, task_id: str):
        """禁用任务"""
        if task_id in self._tasks:
            self._tasks[task_id].enabled = False
            logger.info(f"禁用任务: {task_id}")
    
    def get_task(self, task_id: str) -> Optional[ScheduledTask]:
        """获取任务"""
        return self._tasks.get(task_id)
    
    def list_tasks(self) -> List[ScheduledTask]:
        """列出所有任务"""
        return list(self._tasks.values())
    
    async def run_task(self, task_id: str) -> bool:
        """
        手动运行任务
        
        Args:
            task_id: 任务ID
            
        Returns:
            bool: 是否成功
        """
        task = self._tasks.get(task_id)
        if not task:
            logger.error(f"任务不存在: {task_id}")
            return False
        
        return await self._execute_task(task)
    
    async def _execute_task(self, task: ScheduledTask) -> bool:
        """执行任务"""
        logger.info(f"执行任务: {task.name}")
        
        task.status = TaskStatus.RUNNING
        task.last_run = datetime.now()
        
        try:
            if asyncio.iscoroutinefunction(task.handler):
                await task.handler(**task.params)
            else:
                task.handler(**task.params)
            
            task.status = TaskStatus.COMPLETED
            task.run_count += 1
            task.error = None
            
            logger.info(f"任务完成: {task.name}")
            return True
            
        except Exception as e:
            task.status = TaskStatus.FAILED
            task.error = str(e)
            logger.error(f"任务失败: {task.name}, error={e}")
            return False
        
        finally:
            # 计算下次运行时间
            task.next_run = task.calculate_next_run()
    
    async def start(self):
        """启动调度器"""
        self._running = True
        logger.info("任务调度器已启动")
        
        while self._running:
            now = datetime.now()
            
            for task in self._tasks.values():
                if not task.enabled:
                    continue
                
                if task.next_run and task.next_run <= now:
                    # 执行任务
                    asyncio.create_task(self._execute_task(task))
            
            # 每秒检查一次
            await asyncio.sleep(1)
    
    def stop(self):
        """停止调度器"""
        self._running = False
        logger.info("任务调度器已停止")


class IntervalTask:
    """间隔任务"""
    
    def __init__(
        self,
        name: str,
        interval_seconds: float,
        handler: Callable,
        params: Dict[str, Any] = None,
    ):
        self.name = name
        self.interval_seconds = interval_seconds
        self.handler = handler
        self.params = params or {}
        self._running = False
        self._task: Optional[asyncio.Task] = None
    
    async def start(self):
        """启动间隔任务"""
        self._running = True
        self._task = asyncio.create_task(self._run_loop())
        logger.info(f"间隔任务已启动: {self.name}")
    
    async def stop(self):
        """停止间隔任务"""
        self._running = False
        if self._task:
            self._task.cancel()
        logger.info(f"间隔任务已停止: {self.name}")
    
    async def _run_loop(self):
        """运行循环"""
        while self._running:
            try:
                if asyncio.iscoroutinefunction(self.handler):
                    await self.handler(**self.params)
                else:
                    self.handler(**self.params)
            except Exception as e:
                logger.error(f"间隔任务执行失败: {self.name}, error={e}")
            
            await asyncio.sleep(self.interval_seconds)


class DelayedTask:
    """延迟任务"""
    
    @staticmethod
    async def run_after(
        delay_seconds: float,
        handler: Callable,
        params: Dict[str, Any] = None,
    ):
        """延迟执行"""
        await asyncio.sleep(delay_seconds)
        
        if asyncio.iscoroutinefunction(handler):
            await handler(**(params or {}))
        else:
            handler(**(params or {}))
    
    @staticmethod
    async def run_at(
        target_time: datetime,
        handler: Callable,
        params: Dict[str, Any] = None,
    ):
        """在指定时间执行"""
        now = datetime.now()
        delay = (target_time - now).total_seconds()
        
        if delay > 0:
            await DelayedTask.run_after(delay, handler, params)
        else:
            logger.warning(f"目标时间已过: {target_time}")


# 常用cron表达式
CRON_EXPRESSIONS = {
    "every_minute": "* * * * *",
    "every_hour": "0 * * * *",
    "every_day_9am": "0 9 * * *",
    "every_day_6pm": "0 18 * * *",
    "every_monday_9am": "0 9 * * 1",
    "every_friday_5pm": "0 17 * * 5",
    "first_day_of_month": "0 9 1 * *",
    "every_weekday_9am": "0 9 * * 1-5",
}



"""
测试：Playwright-EZ 基础功能
"""
import asyncio
import pytest
import sys
sys.path.insert(0, "/root/.openclaw/workspace/projects/playwright-ez/src")

from playwright_ez import EasyBrowser, Config, WaitCondition


@pytest.mark.asyncio
async def test_browser_start_close():
    """测试浏览器启动和关闭"""
    browser = EasyBrowser()
    
    # 启动
    await browser.start()
    assert browser._browser is not None
    assert len(browser._tabs) == 1
    
    # 关闭
    await browser.close()
    assert browser._browser is None
    assert len(browser._tabs) == 0


@pytest.mark.asyncio
async def test_context_manager():
    """测试上下文管理器"""
    async with EasyBrowser() as browser:
        assert browser._browser is not None
        assert len(browser._tabs) == 1


@pytest.mark.asyncio
async def test_multi_tab():
    """测试多标签页"""
    async with EasyBrowser() as browser:
        # 创建新标签页
        tab1 = await browser.new_tab("tab1")
        assert tab1.name == "tab1"
        assert len(browser._tabs) == 2
        
        tab2 = await browser.new_tab("tab2")
        assert len(browser._tabs) == 3
        
        # 切换标签页
        browser.switch_tab(tab1.id)
        assert browser._current_tab == tab1.id
        
        # 关闭标签页
        await browser.close_tab(tab1.id)
        assert len(browser._tabs) == 2


@pytest.mark.asyncio
async def test_navigation():
    """测试导航"""
    async with EasyBrowser() as browser:
        await browser.goto("https://www.baidu.com")
        tab = browser.get_tab()
        assert "baidu.com" in tab.url


@pytest.mark.asyncio
async def test_click_and_type():
    """测试点击和输入"""
    async with EasyBrowser() as browser:
        await browser.goto("https://www.baidu.com")

        # 获取页面标题
        title = await browser.page.title()
        assert "百度" in title


@pytest.mark.asyncio
async def test_smart_wait():
    """测试智能等待"""
    async with EasyBrowser() as browser:
        await browser.goto("https://www.baidu.com")

        # 使用更稳定的元素 - 页面本身
        locator = browser.page.locator("body")

        # 等待元素附加到DOM
        result = await browser.smart_wait.wait_for(
            locator,
            WaitCondition.ATTACHED,
            timeout=10
        )
        assert result.success


@pytest.mark.asyncio
async def test_screenshot():
    """测试截图"""
    async with EasyBrowser() as browser:
        await browser.goto("https://www.baidu.com")
        
        screenshot = await browser.screenshot("/tmp/test_screenshot.png")
        assert screenshot is not None
        assert len(screenshot) > 0


@pytest.mark.asyncio
async def test_parallel_action():
    """测试并行操作"""
    async with EasyBrowser() as browser:
        # 创建多个标签页
        tab1 = await browser.new_tab("tab1")
        tab2 = await browser.new_tab("tab2")
        
        # 并行执行
        async def get_url(tab):
            return tab.url
        
        results = await browser.parallel_action(get_url)
        assert len(results) == 3  # default + tab1 + tab2


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""
配置模块
"""
from dataclasses import dataclass, field
from typing import Optional
from loguru import logger
import sys


@dataclass
class Config:
    """Playwright-EZ 配置"""

    # 浏览器配置
    headless: bool = True
    browser_type: str = "chromium"  # chromium, firefox, webkit

    # 窗口配置
    maximize_window: bool = False  # 启动时最大化窗口
    viewport_width: Optional[int] = None  # 视口宽度
    viewport_height: Optional[int] = None  # 视口高度

    # 弹窗配置
    auto_close_popup: bool = False  # 自动关闭弹窗
    popup_close_timeout: float = 5.0  # 弹窗关闭超时（秒）

    # 高亮配置
    highlight_enabled: bool = True
    highlight_color: str = "red"
    highlight_duration: float = 0.5  # 秒
    highlight_style: str = "2px solid {color}"

    # 等待配置
    default_timeout: float = 30.0  # 秒
    poll_interval: float = 0.1  # 轮询间隔
    stability_threshold: int = 3  # 元素稳定性阈值（连续N次检测到才算稳定）

    # 日志配置
    log_enabled: bool = True
    log_level: str = "DEBUG"
    log_format: str = "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | <level>{level: <8}</level> | <cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level>"

    # 多标签配置
    max_tabs: int = 10  # 最大标签页数量

    # 截图配置
    screenshot_on_error: bool = True
    screenshot_dir: str = "./screenshots"

    # 反检测配置
    stealth_enabled: bool = True  # 默认启用反检测
    stealth_user_agent: Optional[str] = None  # 自定义UA（None则随机）
    stealth_locale: Optional[str] = None  # 自定义语言（None则随机）
    stealth_timezone: Optional[str] = None  # 自定义时区（None则随机）
    stealth_viewport: Optional[dict] = None  # 自定义视口（None则随机）

    # 人类化操作配置
    human_behavior_enabled: bool = False  # 默认关闭人类化操作
    human_type_delay_min: int = 50  # 打字最小延迟(毫秒)
    human_type_delay_max: int = 150  # 打字最大延迟(毫秒)
    human_mouse_steps: int = 15  # 鼠标移动步数
    human_scroll_steps: int = 8  # 滚动步数

    def setup_logger(self):
        """配置日志器"""
        logger.remove()
        if self.log_enabled:
            logger.add(
                sys.stderr,
                format=self.log_format,
                level=self.log_level,
                colorize=True,
            )
            logger.add(
                "playwright_ez.log",
                format=self.log_format,
                level=self.log_level,
                rotation="10 MB",
                retention="7 days",
            )
        return logger


# 全局配置实例
_global_config: Optional[Config] = None


def get_config() -> Config:
    """获取全局配置"""
    global _global_config
    if _global_config is None:
        _global_config = Config()
        _global_config.setup_logger()
    return _global_config


def set_config(config: Config):
    """设置全局配置"""
    global _global_config
    _global_config = config
    _global_config.setup_logger()



"""
可访问性检查器
基于 WCAG 标准的可访问性检测
"""
from dataclasses import dataclass, field
from typing import Optional, List, Literal
from playwright.async_api import Page, Locator
from loguru import logger


ImpactLevel = Literal["minor", "moderate", "serious", "critical"]


@dataclass
class AccessibilityNode:
    """可访问性节点"""
    html: str
    target: List[str]
    failure_summary: str = ""


@dataclass
class AccessibilityViolation:
    """可访问性违规"""
    id: str
    impact: ImpactLevel
    description: str
    help: str
    help_url: str
    nodes: List[AccessibilityNode] = field(default_factory=list)


@dataclass
class AccessibilityPass:
    """可访问性通过项"""
    id: str
    description: str
    node_count: int = 0


@dataclass
class AccessibilityCheckResult:
    """可访问性检查结果"""
    passes: bool
    violations: List[AccessibilityViolation] = field(default_factory=list)
    incomplete: List[AccessibilityViolation] = field(default_factory=list)
    passes_detail: List[AccessibilityPass] = field(default_factory=list)


@dataclass
class AccessibilityCheckConfig:
    """可访问性检查配置"""
    impact_threshold: ImpactLevel = "minor"
    rules: Optional[List[str]] = None
    exclude_rules: Optional[List[str]] = None


class AccessibilityChecker:
    """
    可访问性检查器
    基于 WCAG 标准进行可访问性检测
    """

    SEVERITY_ORDER = ["minor", "moderate", "serious", "critical"]

    def __init__(
        self,
        page: Page,
        config: Optional[AccessibilityCheckConfig] = None
    ):
        self.page = page
        self.config = config or AccessibilityCheckConfig()

    async def check(self, selector: Optional[str] = None) -> AccessibilityCheckResult:
        """
        执行可访问性检查

        Args:
            selector: 要检查的元素选择器，默认检查整个页面

        Returns:
            AccessibilityCheckResult: 检查结果
        """
        target = self.page.locator(selector) if selector else self.page.locator("body")

        # 执行各项检查
        results = await asyncio.gather(
            self._check_images(target),
            self._check_labels(target),
            self._check_headings(target),
            self._check_links(target),
            self._check_forms(target),
            self._check_aria(target),
            self._check_color_contrast(target),
            self._check_keyboard(target),
        )

        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []
        incomplete: List[AccessibilityViolation] = []

        for result in results:
            violations.extend(result["violations"])
            passes.extend(result["passes"])
            incomplete.extend(result["incomplete"])

        # 过滤规则
        if self.config.rules:
            rule_set = set(self.config.rules)
            violations = [v for v in violations if v.id in rule_set]
            passes = [p for p in passes if p.id in rule_set]

        if self.config.exclude_rules:
            exclude_set = set(self.config.exclude_rules)
            violations = [v for v in violations if v.id not in exclude_set]
            passes = [p for p in passes if p.id not in exclude_set]

        # 按严重程度过滤
        if self.config.impact_threshold:
            threshold_index = self.SEVERITY_ORDER.index(self.config.impact_threshold)
            violations = [
                v for v in violations
                if self.SEVERITY_ORDER.index(v.impact) >= threshold_index
            ]

        logger.debug(f"可访问性检查完成: violations={len(violations)}, passes={len(passes)}")

        return AccessibilityCheckResult(
            passes=len(violations) == 0,
            violations=violations,
            incomplete=incomplete,
            passes_detail=passes,
        )

    async def _check_images(self, target: Locator) -> dict:
        """检查图片"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        images = target.locator("img")
        count = await images.count()

        pass_count = 0
        for i in range(count):
            img = images.nth(i)
            alt = await img.get_attribute("alt")
            aria_label = await img.get_attribute("aria-label")
            aria_hidden = await img.get_attribute("aria-hidden")

            if alt is not None or aria_label or aria_hidden == "true":
                pass_count += 1
            else:
                html = await img.evaluate("el => el.outerHTML")
                violations.append(AccessibilityViolation(
                    id="image-alt",
                    impact="serious",
                    description="图片缺少替代文本",
                    help="为图片添加 alt 属性描述图片内容",
                    help_url="https://dequeuniversity.com/rules/axe/4.4/image-alt",
                    nodes=[AccessibilityNode(
                        html=html,
                        target=[f"img:nth-of-type({i + 1})"],
                        failure_summary="图片缺少 alt 属性",
                    )],
                ))

        if pass_count > 0:
            passes.append(AccessibilityPass(
                id="image-alt",
                description="图片有替代文本",
                node_count=pass_count,
            ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    async def _check_labels(self, target: Locator) -> dict:
        """检查标签"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        inputs = target.locator('input:not([type="hidden"]):not([type="submit"]):not([type="button"])')
        count = await inputs.count()

        pass_count = 0
        for i in range(count):
            input_el = inputs.nth(i)
            input_id = await input_el.get_attribute("id")
            aria_label = await input_el.get_attribute("aria-label")
            aria_labelledby = await input_el.get_attribute("aria-labelledby")

            has_label = False

            if aria_label or aria_labelledby:
                has_label = True
            elif input_id:
                label = target.locator(f'label[for="{input_id}"]')
                label_count = await label.count()
                has_label = label_count > 0

            if has_label:
                pass_count += 1
            else:
                html = await input_el.evaluate("el => el.outerHTML")
                violations.append(AccessibilityViolation(
                    id="label",
                    impact="serious",
                    description="表单元素缺少关联标签",
                    help="为表单元素添加 label 或 aria-label",
                    help_url="https://dequeuniversity.com/rules/axe/4.4/label",
                    nodes=[AccessibilityNode(
                        html=html,
                        target=[f"input:nth-of-type({i + 1})"],
                        failure_summary="表单元素没有关联的标签",
                    )],
                ))

        if pass_count > 0:
            passes.append(AccessibilityPass(
                id="label",
                description="表单元素有标签",
                node_count=pass_count,
            ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    async def _check_headings(self, target: Locator) -> dict:
        """检查标题层级"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        headings = target.locator("h1, h2, h3, h4, h5, h6")
        count = await headings.count()

        prev_level = 0
        pass_count = 0

        for i in range(count):
            heading = headings.nth(i)
            tag_name = await heading.evaluate("el => el.tagName.toLowerCase()")
            level = int(tag_name[1])

            if prev_level > 0 and level > prev_level + 1:
                html = await heading.evaluate("el => el.outerHTML")
                violations.append(AccessibilityViolation(
                    id="heading-order",
                    impact="moderate",
                    description="标题层级跳跃",
                    help="确保标题层级连续",
                    help_url="https://dequeuniversity.com/rules/axe/4.4/heading-order",
                    nodes=[AccessibilityNode(
                        html=html,
                        target=[f"{tag_name}:nth-of-type({i + 1})"],
                        failure_summary=f"标题从 h{prev_level} 跳到 h{level}",
                    )],
                ))
            else:
                pass_count += 1

            prev_level = level

        if pass_count > 0:
            passes.append(AccessibilityPass(
                id="heading-order",
                description="标题层级正确",
                node_count=pass_count,
            ))

        # 检查页面是否有 h1
        h1_count = await target.locator("h1").count()
        if h1_count == 0:
            violations.append(AccessibilityViolation(
                id="page-has-heading-one",
                impact="moderate",
                description="页面缺少一级标题",
                help="为页面添加唯一的 h1 标题",
                help_url="https://dequeuniversity.com/rules/axe/4.4/page-has-heading-one",
                nodes=[],
            ))
        elif h1_count >= 1:
            passes.append(AccessibilityPass(
                id="page-has-heading-one",
                description="页面有一级标题",
                node_count=h1_count,
            ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    async def _check_links(self, target: Locator) -> dict:
        """检查链接"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        links = target.locator("a[href]")
        count = await links.count()

        pass_count = 0
        for i in range(count):
            link = links.nth(i)
            text = await link.text_content()
            aria_label = await link.get_attribute("aria-label")
            title = await link.get_attribute("title")

            if (text and text.strip()) or aria_label or title:
                pass_count += 1
            else:
                html = await link.evaluate("el => el.outerHTML")
                violations.append(AccessibilityViolation(
                    id="link-name",
                    impact="serious",
                    description="链接没有可识别的文本",
                    help="为链接添加可见文本或 aria-label",
                    help_url="https://dequeuniversity.com/rules/axe/4.4/link-name",
                    nodes=[AccessibilityNode(
                        html=html,
                        target=[f"a:nth-of-type({i + 1})"],
                        failure_summary="链接缺少可识别文本",
                    )],
                ))

        if pass_count > 0:
            passes.append(AccessibilityPass(
                id="link-name",
                description="链接有可识别文本",
                node_count=pass_count,
            ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    async def _check_forms(self, target: Locator) -> dict:
        """检查表单"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        forms = target.locator("form")
        form_count = await forms.count()

        pass_count = 0
        for i in range(form_count):
            form = forms.nth(i)
            submit_button = form.locator('button[type="submit"], input[type="submit"]')
            has_submit = await submit_button.count() > 0

            if has_submit:
                pass_count += 1
            else:
                html = await form.evaluate("el => el.outerHTML.substring(0, 200)")
                violations.append(AccessibilityViolation(
                    id="form-submit",
                    impact="moderate",
                    description="表单缺少提交按钮",
                    help="为表单添加提交按钮",
                    help_url="https://dequeuniversity.com/rules/axe/4.4/label",
                    nodes=[AccessibilityNode(
                        html=html,
                        target=[f"form:nth-of-type({i + 1})"],
                        failure_summary="表单没有提交按钮",
                    )],
                ))

        if pass_count > 0:
            passes.append(AccessibilityPass(
                id="form-submit",
                description="表单有提交按钮",
                node_count=pass_count,
            ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    async def _check_aria(self, target: Locator) -> dict:
        """检查 ARIA 属性"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        hidden_elements = target.locator('[aria-hidden="true"]')
        hidden_count = await hidden_elements.count()

        for i in range(hidden_count):
            element = hidden_elements.nth(i)
            focusable = element.locator("a, button, input, select, textarea, [tabindex]")
            focusable_count = await focusable.count()

            if focusable_count > 0:
                html = await element.evaluate("el => el.outerHTML.substring(0, 200)")
                violations.append(AccessibilityViolation(
                    id="aria-hidden-focus",
                    impact="serious",
                    description="隐藏元素包含可聚焦元素",
                    help="确保 aria-hidden 元素内的交互元素不可聚焦",
                    help_url="https://dequeuniversity.com/rules/axe/4.4/aria-hidden-focus",
                    nodes=[AccessibilityNode(
                        html=html,
                        target=[f'[aria-hidden="true"]:nth-of-type({i + 1})'],
                        failure_summary="隐藏元素包含可聚焦的子元素",
                    )],
                ))

        passes.append(AccessibilityPass(
            id="aria-valid-attr",
            description="ARIA 属性有效",
            node_count=hidden_count,
        ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    async def _check_color_contrast(self, target: Locator) -> dict:
        """检查颜色对比度（简化版）"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        styled_elements = target.locator('[style*="color"], [style*="background"]')
        count = await styled_elements.count()

        # 实际项目中应该使用专门的对比度计算库
        passes.append(AccessibilityPass(
            id="color-contrast",
            description="颜色对比度检查（需手动验证）",
            node_count=count,
        ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    async def _check_keyboard(self, target: Locator) -> dict:
        """检查键盘可访问性"""
        violations: List[AccessibilityViolation] = []
        passes: List[AccessibilityPass] = []

        click_elements = target.locator("[onclick]")
        count = await click_elements.count()

        pass_count = 0
        for i in range(count):
            element = click_elements.nth(i)
            tabindex = await element.get_attribute("tabindex")
            tag_name = await element.evaluate("el => el.tagName.toLowerCase()")

            is_focusable = tag_name in ["a", "button", "input", "select", "textarea"] or tabindex is not None

            if is_focusable:
                pass_count += 1
            else:
                html = await element.evaluate("el => el.outerHTML.substring(0, 200)")
                violations.append(AccessibilityViolation(
                    id="keyboard",
                    impact="serious",
                    description="可点击元素无法通过键盘访问",
                    help="为元素添加 tabindex 或使用 button/a 元素",
                    help_url="https://dequeuniversity.com/rules/axe/4.4/keyboard",
                    nodes=[AccessibilityNode(
                        html=html,
                        target=[f"[onclick]:nth-of-type({i + 1})"],
                        failure_summary="元素可通过鼠标点击但无法通过键盘聚焦",
                    )],
                ))

        if pass_count > 0:
            passes.append(AccessibilityPass(
                id="keyboard",
                description="可点击元素可通过键盘访问",
                node_count=pass_count,
            ))

        return {"violations": violations, "passes": passes, "incomplete": []}

    def generate_report(self, result: AccessibilityCheckResult) -> str:
        """
        生成报告

        Args:
            result: 检查结果

        Returns:
            str: 报告文本
        """
        lines = [
            "\n=== 可访问性检查报告 ===\n",
            f"状态: {'✅ 通过' if result.passes else '❌ 未通过'}",
            f"违规项: {len(result.violations)}",
            f"不完整项: {len(result.incomplete)}",
            f"通过项: {len(result.passes_detail)}\n",
        ]

        if result.violations:
            lines.append("违规详情:")
            for v in result.violations:
                lines.extend([
                    f"\n[{v.impact.upper()}] {v.id}",
                    f"  {v.description}",
                    f"  建议: {v.help}",
                    f"  文档: {v.help_url}",
                ])
                if v.nodes:
                    lines.append(f"  影响元素: {len(v.nodes)} 个")

        return "\n".join(lines)


import asyncio


def create_accessibility_checker(
    page: Page,
    config: Optional[AccessibilityCheckConfig] = None
) -> AccessibilityChecker:
    """创建可访问性检查器实例"""
    return AccessibilityChecker(page, config)



"""
诊断模块
提供控制台监控、错误分析、页面快照等功能
"""
from .console_monitor import (
    ConsoleMonitor,
    ConsoleLogEntry,
    ConsoleMonitorConfig,
    ConsoleMonitorResult,
    ConsoleFilterConfig,
    create_console_monitor,
)
from .error_analyzer import (
    ErrorAnalyzer,
    ErrorAnalysisResult,
    ErrorLocation,
    create_error_analyzer,
)
from .page_snapshot import (
    PageSnapshot,
    SnapshotConfig,
    SnapshotResult,
    DOMSnapshot,
    DiffResult,
    create_page_snapshot,
)

__all__ = [
    # 控制台监控
    "ConsoleMonitor",
    "ConsoleLogEntry",
    "ConsoleMonitorConfig",
    "ConsoleMonitorResult",
    "ConsoleFilterConfig",
    "create_console_monitor",
    # 错误分析
    "ErrorAnalyzer",
    "ErrorAnalysisResult",
    "ErrorLocation",
    "create_error_analyzer",
    # 页面快照
    "PageSnapshot",
    "SnapshotConfig",
    "SnapshotResult",
    "DOMSnapshot",
    "DiffResult",
    "create_page_snapshot",
]



"""
安全辅助类
整合 security 子模块，提供统一访问入口
"""
from typing import Callable, Optional
from playwright.async_api import Page

from ..security import (
    SecurityScanner,
    XSSDetector,
    XSSConfig,
    HeaderChecker,
    HeaderCheckConfig,
)


class SecurityHelper:
    """
    安全辅助类
    整合 security 子模块

    使用方式:
        browser.security.scanner  # SecurityScanner
        browser.security.xss      # XSSDetector
        browser.security.header   # HeaderChecker
    """

    def __init__(self, get_page: Callable[[], Page]):
        """
        初始化安全辅助类

        Args:
            get_page: 获取当前 Page 对象的回调函数
        """
        self._get_page = get_page
        self._scanner: Optional[SecurityScanner] = None
        self._xss: Optional[XSSDetector] = None
        self._header: Optional[HeaderChecker] = None
        self._xss_config: Optional[XSSConfig] = None
        self._header_config: Optional[HeaderCheckConfig] = None

    @property
    def scanner(self) -> SecurityScanner:
        """
        获取安全扫描器

        Returns:
            SecurityScanner: 安全扫描器实例
        """
        if self._scanner is None:
            self._scanner = SecurityScanner(self._get_page())
        return self._scanner

    @property
    def xss(self) -> XSSDetector:
        """
        获取 XSS 检测器

        Returns:
            XSSDetector: XSS 检测器实例
        """
        if self._xss is None:
            self._xss = XSSDetector(self._get_page(), self._xss_config)
        return self._xss

    @property
    def header(self) -> HeaderChecker:
        """
        获取 Header 检查器

        Returns:
            HeaderChecker: Header 检查器实例
        """
        if self._header is None:
            self._header = HeaderChecker(self._get_page(), self._header_config)
        return self._header

    def set_xss_config(self, config: XSSConfig):
        """设置 XSS 检测配置"""
        self._xss_config = config
        self._xss = None

    def set_header_config(self, config: HeaderCheckConfig):
        """设置 Header 检查配置"""
        self._header_config = config
        self._header = None

    def reset(self):
        """
        重置所有实例
        当切换标签页时调用，确保使用正确的 Page 对象
        """
        self._scanner = None
        self._xss = None
        self._header = None



"""
测试：引擎模块
测试规则引擎和混合引擎
"""
import pytest
from unittest.mock import AsyncMock, MagicMock, patch
import json

from playwright_ez.ai.types import AIConfig, LLMConfig, LLMProvider, SelectorSuggestion
from playwright_ez.ai.engine.rule_engine import RuleEngine
from playwright_ez.ai.engine.hybrid_engine import HybridEngine


class MockPage:
    """模拟 Page 对象"""

    def __init__(self):
        self.url = "https://example.com"
        self._locator = MagicMock()

    def locator(self, selector: str):
        return self._locator


class TestRuleEngine:
    """测试规则引擎"""

    @pytest.fixture
    def mock_page(self):
        """创建模拟页面"""
        page = MockPage()
        # 配置 locator 的 count 方法返回 1
        page._locator.count = AsyncMock(return_value=1)
        return page

    @pytest.fixture
    def rule_engine(self, mock_page):
        """创建规则引擎实例"""
        return RuleEngine(mock_page)

    @pytest.mark.asyncio
    async def test_suggest_selectors_for_button(self, rule_engine, mock_page):
        """测试按钮选择器建议"""
        suggestions = await rule_engine.suggest_selectors("登录按钮", "button")

        assert len(suggestions) > 0
        assert all(isinstance(s, SelectorSuggestion) for s in suggestions)
        # 检查是否有 button 相关的选择器
        selectors = [s.selector for s in suggestions]
        assert any("button" in s for s in selectors)

    @pytest.mark.asyncio
    async def test_suggest_selectors_for_input(self, rule_engine, mock_page):
        """测试输入框选择器建议"""
        suggestions = await rule_engine.suggest_selectors("用户名", "input")

        assert len(suggestions) > 0
        # 检查是否有 input 相关的选择器
        selectors = [s.selector for s in suggestions]
        assert any("input" in s or "placeholder" in s for s in selectors)

    @pytest.mark.asyncio
    async def test_suggest_selectors_for_link(self, rule_engine, mock_page):
        """测试链接选择器建议"""
        suggestions = await rule_engine.suggest_selectors("关于我们", "link")

        assert len(suggestions) > 0
        selectors = [s.selector for s in suggestions]
        assert any("a:" in s or "a[" in s for s in selectors)

    @pytest.mark.asyncio
    async def test_suggest_selectors_generic(self, rule_engine, mock_page):
        """测试通用选择器建议"""
        suggestions = await rule_engine.suggest_selectors("测试文本")

        assert len(suggestions) > 0
        # 应该有文本相关的选择器
        selectors = [s.selector for s in suggestions]
        assert any("text" in s.lower() for s in selectors)

    @pytest.mark.asyncio
    async def test_suggest_selectors_confidence_ordering(self, rule_engine, mock_page):
        """测试选择器按置信度排序"""
        # 配置多个元素匹配
        mock_page._locator.count = AsyncMock(return_value=1)

        suggestions = await rule_engine.suggest_selectors("按钮")

        # 检查是否按置信度降序排列
        if len(suggestions) > 1:
            for i in range(len(suggestions) - 1):
                assert suggestions[i].confidence >= suggestions[i + 1].confidence

    @pytest.mark.asyncio
    async def test_suggest_selectors_max_suggestions(self, mock_page):
        """测试最大建议数量限制"""
        config = AIConfig(max_suggestions=3)
        engine = RuleEngine(mock_page, config)

        suggestions = await engine.suggest_selectors("按钮")
        assert len(suggestions) <= 3

    @pytest.mark.asyncio
    async def test_diagnose_selector_not_found(self, rule_engine):
        """测试选择器未找到错误诊断"""
        error = Exception("Element not found: #missing")

        diagnosis = await rule_engine.diagnose_error(error)

        assert diagnosis.error_type == "selector_not_found"
        assert len(diagnosis.likely_causes) > 0
        assert len(diagnosis.suggested_solutions) > 0

    @pytest.mark.asyncio
    async def test_diagnose_timeout_error(self, rule_engine):
        """测试超时错误诊断"""
        error = Exception("Timeout waiting for element")

        diagnosis = await rule_engine.diagnose_error(error)

        assert diagnosis.error_type == "timeout"
        assert len(diagnosis.likely_causes) > 0
        assert len(diagnosis.suggested_solutions) > 0

    @pytest.mark.asyncio
    async def test_diagnose_navigation_error(self, rule_engine):
        """测试导航错误诊断"""
        error = Exception("Navigation failed to https://example.com")

        diagnosis = await rule_engine.diagnose_error(error)

        assert diagnosis.error_type == "navigation"

    @pytest.mark.asyncio
    async def test_diagnose_unknown_error(self, rule_engine):
        """测试未知错误诊断"""
        error = Exception("Something went wrong")

        diagnosis = await rule_engine.diagnose_error(error)

        assert diagnosis.error_type == "unknown"


class TestHybridEngine:
    """测试混合引擎"""

    @pytest.fixture
    def mock_page(self):
        """创建模拟页面"""
        page = MockPage()
        page._locator.count = AsyncMock(return_value=1)
        return page

    @pytest.fixture
    def rule_config(self):
        """纯规则引擎配置"""
        return AIConfig(
            llm=LLMConfig(provider=LLMProvider.RULE)
        )

    @pytest.fixture
    def custom_llm_config(self):
        """自定义 LLM 配置"""
        return AIConfig(
            llm=LLMConfig(
                provider=LLMProvider.CUSTOM,
                custom_call_fn=lambda x: json.dumps([
                    {"selector": "button.submit", "confidence": 0.9, "strategy": "class"}
                ])
            )
        )

    @pytest.mark.asyncio
    async def test_rule_mode(self, mock_page, rule_config):
        """测试纯规则模式"""
        engine = HybridEngine(mock_page, rule_config)

        suggestions = await engine.suggest_selectors("登录按钮")

        assert len(suggestions) > 0
        # 规则引擎生成的建议有特定格式
        assert all(s.strategy != "llm" for s in suggestions)

    @pytest.mark.asyncio
    async def test_custom_llm_mode(self, mock_page, custom_llm_config):
        """测试自定义 LLM 模式"""
        engine = HybridEngine(mock_page, custom_llm_config)

        suggestions = await engine.suggest_selectors("提交按钮")

        # 自定义 LLM 返回了有效的选择器
        assert len(suggestions) > 0
        assert suggestions[0].selector == "button.submit"

    @pytest.mark.asyncio
    async def test_fallback_to_rule(self, mock_page):
        """测试 LLM 失败时回退到规则引擎"""
        # 配置一个会失败的 LLM
        def failing_llm(prompt: str) -> str:
            raise Exception("LLM failed")

        config = AIConfig(
            llm=LLMConfig(
                provider=LLMProvider.CUSTOM,
                custom_call_fn=failing_llm
            ),
            fallback_to_rule=True
        )

        engine = HybridEngine(mock_page, config)

        suggestions = await engine.suggest_selectors("按钮")

        # 应该回退到规则引擎并返回结果
        assert len(suggestions) > 0

    @pytest.mark.asyncio
    async def test_fallback_disabled(self, mock_page):
        """测试禁用回退时 LLM 失败返回空"""
        def failing_llm(prompt: str) -> str:
            raise Exception("LLM failed")

        config = AIConfig(
            llm=LLMConfig(
                provider=LLMProvider.CUSTOM,
                custom_call_fn=failing_llm
            ),
            fallback_to_rule=False
        )

        engine = HybridEngine(mock_page, config)

        suggestions = await engine.suggest_selectors("按钮")

        # 禁用回退，LLM 失败时应返回空
        assert len(suggestions) == 0

    @pytest.mark.asyncio
    async def test_llm_json_parsing_with_markdown(self, mock_page):
        """测试 LLM 返回 markdown 格式的 JSON"""
        def llm_with_markdown(prompt: str) -> str:
            return """```json
[{"selector": "#submit-btn", "confidence": 0.95, "strategy": "id"}]
```"""

        config = AIConfig(
            llm=LLMConfig(
                provider=LLMProvider.CUSTOM,
                custom_call_fn=llm_with_markdown
            )
        )

        engine = HybridEngine(mock_page, config)

        suggestions = await engine.suggest_selectors("提交")

        assert len(suggestions) > 0
        assert suggestions[0].selector == "#submit-btn"

    @pytest.mark.asyncio
    async def test_llm_diagnose_error(self, mock_page, custom_llm_config):
        """测试 LLM 错误诊断"""
        engine = HybridEngine(mock_page, custom_llm_config)

        # 修改 LLM 返回诊断结果
        custom_llm_config.llm.custom_call_fn = lambda x: json.dumps({
            "error_type": "selector_issue",
            "likely_causes": ["元素不存在", "选择器错误"],
            "suggested_solutions": ["检查选择器", "添加等待"],
            "related_elements": []
        })

        error = Exception("Element not found")
        diagnosis = await engine.diagnose_error(error)

        assert diagnosis.error_type == "selector_issue"
        assert len(diagnosis.likely_causes) > 0

    @pytest.mark.asyncio
    async def test_use_llm_first_disabled(self, mock_page):
        """测试禁用 LLM 优先"""
        config = AIConfig(
            llm=LLMConfig(
                provider=LLMProvider.CUSTOM,
                custom_call_fn=lambda x: "[]"
            ),
            use_llm_first=False
        )

        engine = HybridEngine(mock_page, config)

        suggestions = await engine.suggest_selectors("按钮")

        # 禁用 LLM 优先，直接使用规则引擎
        assert len(suggestions) > 0


class TestAIConfig:
    """测试 AIConfig 配置类"""

    def test_default_values(self):
        """测试默认值"""
        config = AIConfig()

        assert config.enable_healing is True
        assert config.enable_suggestions is True
        assert config.max_suggestions == 5
        assert config.use_llm_first is True
        assert config.fallback_to_rule is True
        assert config.llm.provider == LLMProvider.RULE

    def test_custom_llm_config(self):
        """测试自定义 LLM 配置"""
        config = AIConfig(
            llm=LLMConfig(
                provider=LLMProvider.CLAUDE,
                claude_api_key="sk-test"
            )
        )

        assert config.llm.provider == LLMProvider.CLAUDE
        assert config.llm.claude_api_key == "sk-test"

    def test_hybrid_mode_config(self):
        """测试混合模式配置"""
        config = AIConfig(
            use_llm_first=True,
            fallback_to_rule=True,
            llm=LLMConfig(provider=LLMProvider.OPENAI)
        )

        assert config.use_llm_first is True
        assert config.fallback_to_rule is True
        assert config.llm.provider == LLMProvider.OPENAI


if __name__ == "__main__":
    pytest.main([__file__, "-v"])


class TestAIUnifiedAccess:
    """测试 AI 模块统一入口访问"""

    @pytest.mark.asyncio
    async def test_browser_ai_accessor(self):
        """测试 browser.ai 统一访问入口"""
        from playwright_ez import EasyBrowser

        async with EasyBrowser() as browser:
            # 验证 ai 属性存在且类型正确
            from playwright_ez.ai import AIHelper
            assert isinstance(browser.ai, AIHelper)

    @pytest.mark.asyncio
    async def test_browser_ai_suggest_selectors(self):
        """测试通过 browser.ai 进行选择器建议"""
        from playwright_ez import EasyBrowser

        async with EasyBrowser() as browser:
            await browser.goto("https://www.baidu.com")

            # 使用统一入口进行选择器建议
            suggestions = await browser.ai.suggest_selectors("百度一下", "button")

            assert len(suggestions) > 0
            assert all(s.selector for s in suggestions)
            assert all(s.confidence > 0 for s in suggestions)

    @pytest.mark.asyncio
    async def test_browser_ai_diagnose_error(self):
        """测试通过 browser.ai 进行错误诊断"""
        from playwright_ez import EasyBrowser

        async with EasyBrowser() as browser:
            await browser.goto("https://www.baidu.com")

            # 模拟一个错误
            error = Exception("Element not found: #missing-element")
            diagnosis = await browser.ai.diagnose_error(error)

            assert diagnosis.error_type == "selector_not_found"
            assert len(diagnosis.likely_causes) > 0
            assert len(diagnosis.suggested_solutions) > 0

    @pytest.mark.asyncio
    async def test_browser_ai_generate_code(self):
        """测试通过 browser.ai 生成测试代码"""
        from playwright_ez import EasyBrowser

        async with EasyBrowser() as browser:
            await browser.goto("https://www.baidu.com")

            code = await browser.ai.generate_test_code("搜索测试")

            assert code.code is not None
            assert "async def" in code.code
            assert code.language == "python"

    @pytest.mark.asyncio
    async def test_browser_ai_with_custom_config(self):
        """测试带自定义配置的 browser.ai"""
        from playwright_ez import EasyBrowser, AIConfig, LLMConfig, LLMProvider

        # 使用规则引擎配置
        ai_config = AIConfig(
            llm=LLMConfig(provider=LLMProvider.RULE),
            max_suggestions=3
        )

        async with EasyBrowser() as browser:
            # 重新创建 AIHelper 使用自定义配置
            from playwright_ez.ai import AIHelper
            custom_ai = AIHelper(browser.page, ai_config)

            suggestions = await custom_ai.suggest_selectors("按钮")
            assert len(suggestions) <= 3

    @pytest.mark.asyncio
    async def test_ai_accessor_lazy_loading(self):
        """测试 AI 访问器懒加载"""
        from playwright_ez import EasyBrowser

        async with EasyBrowser() as browser:
            # 多次访问应返回同一实例
            ai1 = browser.ai
            ai2 = browser.ai
            assert ai1 is ai2

    @pytest.mark.asyncio
    async def test_ai_accessor_reset_on_tab_switch(self):
        """测试切换标签页时 AI 实例重置"""
        from playwright_ez import EasyBrowser

        async with EasyBrowser() as browser:
            ai1 = browser.ai

            # 创建新标签页
            await browser.new_tab("test_tab")

            # AI 实例应该被重置
            ai2 = browser.ai
            # 注意：这里不一定是同一个实例，因为切换标签页会重置



"""
智能爬虫
自动识别页面结构并提取数据
"""
import json
from typing import Optional, List, Dict, Any
from playwright.async_api import Page
from loguru import logger

from .basic import DataExtractor
from .pagination import PaginationExtractor
from .dynamic import DynamicContentExtractor
from .form import FormDataExtractor


class SmartScraper:
    """
    智能爬虫
    自动识别页面结构并提取数据
    """

    def __init__(self, page: Page):
        self.page = page
        self.extractor = DataExtractor(page)
        self.pagination = PaginationExtractor(page)
        self.dynamic = DynamicContentExtractor(page)
        self.form = FormDataExtractor(page)

    async def auto_detect_list(
        self,
        min_items: int = 3,
        similarity_threshold: float = 0.7,
    ) -> Optional[Dict[str, Any]]:
        """
        自动检测页面中的列表结构

        Args:
            min_items: 最小列表项数量
            similarity_threshold: 相似度阈值

        Returns:
            检测结果
        """
        result = await self.page.evaluate(f"""
            () => {{
                // 查找重复结构的容器
                const candidates = [];

                // 常见列表容器
                const listSelectors = [
                    'ul > li', 'ol > li',
                    'table tbody tr',
                    '.list > div', '.items > div',
                    '[class*="list"] > div', '[class*="item"]',
                    '.row > div', '.grid > div',
                ];

                for (const selector of listSelectors) {{
                    const items = document.querySelectorAll(selector);
                    if (items.length >= {min_items}) {{
                        // 分析第一个和第二个项的结构相似度
                        const item1 = items[0];
                        const item2 = items[1];

                        if (item1 && item2) {{
                            const struct1 = getItemStructure(item1);
                            const struct2 = getItemStructure(item2);
                            const similarity = calculateSimilarity(struct1, struct2);

                            if (similarity >= {similarity_threshold}) {{
                                candidates.push({{
                                    selector: selector,
                                    count: items.length,
                                    similarity: similarity,
                                    structure: struct1,
                                }});
                            }}
                        }}
                    }}
                }}

                // 辅助函数
                function getItemStructure(el) {{
                    return {{
                        children: el.children.length,
                        links: el.querySelectorAll('a').length,
                        images: el.querySelectorAll('img').length,
                        textLength: el.textContent.length,
                        classes: Array.from(el.classList),
                    }};
                }}

                function calculateSimilarity(a, b) {{
                    const keys = ['children', 'links', 'images'];
                    let matches = 0;
                    keys.forEach(k => {{
                        if (a[k] === b[k]) matches++;
                    }});
                    return matches / keys.length;
                }}

                return candidates.sort((a, b) => b.similarity - a.similarity)[0] || null;
            }}
        """)
        return result

    async def smart_extract_list(
        self,
        item_selector: Optional[str] = None,
        fields: Optional[List[str]] = None,
    ) -> List[Dict[str, Any]]:
        """
        智能提取列表数据 - 批量优化版

        Args:
            item_selector: 列表项选择器（自动检测如果为空）
            fields: 要提取的字段名（自动检测如果为空）

        Returns:
            数据列表
        """
        # 自动检测列表选择器
        if not item_selector:
            detected = await self.auto_detect_list()
            if detected:
                item_selector = detected["selector"]
            else:
                logger.warning("无法自动检测列表结构")
                return []

        try:
            # 自动检测字段模式
            if not fields:
                results = await self.page.evaluate(f"""
                    () => {{
                        const items = document.querySelectorAll('{item_selector}');
                        const results = [];

                        items.forEach(item => {{
                            const row = {{}};

                            // 提取文本和 HTML
                            row.text = item.textContent.trim();
                            row.html = item.innerHTML;

                            // 提取链接
                            const links = item.querySelectorAll('a[href]');
                            if (links.length > 0) {{
                                row.links = [];
                                links.forEach(link => {{
                                    row.links.push({{
                                        text: link.textContent.trim(),
                                        href: link.href,
                                    }});
                                }});
                            }}

                            // 提取图片
                            const images = item.querySelectorAll('img[src]');
                            if (images.length > 0) {{
                                row.images = [];
                                images.forEach(img => {{
                                    row.images.push({{
                                        src: img.src,
                                        alt: img.alt || '',
                                    }});
                                }});
                            }}

                            results.push(row);
                        }});

                        return results;
                    }}
                """)
            else:
                # 按指定字段提取
                fields_json = json.dumps(fields)

                results = await self.page.evaluate(f"""
                    () => {{
                        const items = document.querySelectorAll('{item_selector}');
                        const fields = {fields_json};
                        const results = [];

                        items.forEach(item => {{
                            const row = {{}};
                            fields.forEach(field => {{
                                const el = item.querySelector('[class*="' + field + '"], [id*="' + field + '"], [name*="' + field + '"]');
                                row[field] = el ? el.textContent.trim() : null;
                            }});
                            results.push(row);
                        }});

                        return results;
                    }}
                """)

            return results
        except Exception as e:
            logger.error(f"智能提取列表失败: {e}")
            return []

    async def extract_page_summary(self) -> Dict[str, Any]:
        """
        提取页面摘要信息

        Returns:
            页面摘要
        """
        summary = {
            "url": self.page.url,
            "title": await self.page.title(),
        }

        # 提取 meta 信息
        summary["meta"] = await self.extractor.extract_meta_tags()
        summary["open_graph"] = await self.extractor.extract_open_graph()
        summary["json_ld"] = await self.extractor.extract_json_ld()

        # 提取主要文本
        main_content = await self.page.evaluate("""
            () => {
                // 尝试找到主要内容区域
                const main = document.querySelector('main, article, .content, #content, [role="main"]');
                if (main) {
                    return main.textContent.trim().substring(0, 500);
                }
                return document.body.textContent.trim().substring(0, 500);
            }
        """)
        summary["content_preview"] = main_content

        # 统计页面元素
        stats = await self.page.evaluate("""
            () => ({
                headings: document.querySelectorAll('h1, h2, h3').length,
                paragraphs: document.querySelectorAll('p').length,
                links: document.querySelectorAll('a').length,
                images: document.querySelectorAll('img').length,
                forms: document.querySelectorAll('form').length,
                tables: document.querySelectorAll('table').length,
                lists: document.querySelectorAll('ul, ol').length,
            })
        """)
        summary["stats"] = stats

        return summary

    async def extract_all_text(self) -> Dict[str, str]:
        """
        提取页面所有文本内容（按区域）

        Returns:
            区域文本字典
        """
        text_content = await self.page.evaluate("""
            () => {
                const result = {};

                // 标题
                const h1 = document.querySelector('h1');
                if (h1) result.title = h1.textContent.trim();

                // 副标题
                const h2s = document.querySelectorAll('h2');
                if (h2s.length > 0) {
                    result.subtitles = Array.from(h2s).map(h => h.textContent.trim());
                }

                // 段落
                const paragraphs = document.querySelectorAll('p');
                if (paragraphs.length > 0) {
                    result.paragraphs = Array.from(paragraphs).map(p => p.textContent.trim()).filter(t => t);
                }

                // 导航
                const nav = document.querySelector('nav, .nav, .navigation');
                if (nav) result.navigation = nav.textContent.trim();

                // 页脚
                const footer = document.querySelector('footer, .footer');
                if (footer) result.footer = footer.textContent.trim();

                // 侧边栏
                const sidebar = document.querySelector('aside, .sidebar, .side');
                if (sidebar) result.sidebar = sidebar.textContent.trim();

                return result;
            }
        """)
        return text_content

    async def extract_contact_info(self) -> Dict[str, List[str]]:
        """
        提取页面中的联系信息

        Returns:
            联系信息字典
        """
        import re

        page_text = await self.page.evaluate("() => document.body.textContent")
        contact_info = {
            "emails": [],
            "phones": [],
            "addresses": [],
        }

        # 提取邮箱
        email_pattern = r'[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}'
        emails = re.findall(email_pattern, page_text or "")
        contact_info["emails"] = list(set(emails))

        # 提取电话号码
        phone_pattern = r'(?:\+?86[-\s]?)?(?:1[3-9]\d{9}|(?:\d{3,4}[-\s]?)?\d{7,8})'
        phones = re.findall(phone_pattern, page_text or "")
        contact_info["phones"] = list(set(phones))

        return contact_info

    async def extract_social_links(self) -> Dict[str, str]:
        """
        提取社交媒体链接

        Returns:
            社交媒体链接字典
        """
        social_links = await self.page.evaluate("""
            () => {
                const result = {};
                const patterns = {
                    weibo: /weibo\\.com/,
                    wechat: /weixin\\.qq\\.com|wechat/,
                    twitter: /twitter\\.com|x\\.com/,
                    facebook: /facebook\\.com|fb\\.com/,
                    instagram: /instagram\\.com/,
                    linkedin: /linkedin\\.com/,
                    youtube: /youtube\\.com|youtu\\.be/,
                    github: /github\\.com/,
                    tiktok: /tiktok\\.com|douyin\\.com/,
                };

                document.querySelectorAll('a[href]').forEach(a => {
                    const href = a.href;
                    for (const [platform, pattern] of Object.entries(patterns)) {
                        if (pattern.test(href)) {
                            result[platform] = href;
                            break;
                        }
                    }
                });

                return result;
            }
        """)
        return social_links

    async def smart_scrape(
        self,
        url: Optional[str] = None,
        extract_type: str = "auto",
    ) -> Dict[str, Any]:
        """
        智能爬取页面

        Args:
            url: 目标URL（可选，默认当前页面）
            extract_type: 提取类型 auto/list/article/product

        Returns:
            爬取结果
        """
        if url:
            await self.page.goto(url)
            await self.page.wait_for_load_state("networkidle")

        result = {
            "url": self.page.url,
            "title": await self.page.title(),
        }

        # 根据类型选择提取策略
        if extract_type == "auto":
            # 自动判断页面类型
            list_detected = await self.auto_detect_list()
            if list_detected and list_detected.get("count", 0) >= 3:
                result["type"] = "list"
                result["data"] = await self.smart_extract_list()
            else:
                result["type"] = "article"
                result["data"] = await self.extract_all_text()
                result["summary"] = await self.extract_page_summary()

        elif extract_type == "list":
            result["type"] = "list"
            result["data"] = await self.smart_extract_list()

        elif extract_type == "article":
            result["type"] = "article"
            result["data"] = await self.extract_all_text()
            result["summary"] = await self.extract_page_summary()

        else:
            result["type"] = "full"
            result["data"] = await self.extract_all_text()
            result["links"] = await self.extractor.extract_links()
            result["images"] = await self.extractor.extract_images()
            result["summary"] = await self.extract_page_summary()

        return result



"""
SEO 检查器
SEO 基础检查，Meta 标签、标题、结构化数据验证
"""
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any, Callable
from playwright.async_api import Page
from loguru import logger


@dataclass
class SEOCheckResult:
    """SEO 检查结果"""
    id: str
    name: str
    passed: bool
    score: float
    description: str
    details: List[str] = field(default_factory=list)


@dataclass
class SEOReport:
    """SEO 报告"""
    url: str
    total_score: float
    checks: List[SEOCheckResult]
    passed_count: int
    failed_count: int


class SEOChecker:
    """
    SEO 检查器
    检查页面的 SEO 相关指标
    """

    def __init__(self, page: Page):
        self.page = page

    async def check(self) -> SEOReport:
        """
        执行 SEO 检查

        Returns:
            SEOReport: 检查报告
        """
        url = self.page.url

        # 执行各项检查
        checks = [
            await self._check_title(),
            await self._check_meta_description(),
            await self._check_h1(),
            await self._check_images_alt(),
            await self._check_canonical(),
            await self._check_viewport(),
            await self._check_robots(),
            await self._check_og_tags(),
            await self._check_structured_data(),
            await self._check_links(),
        ]

        # 计算总分
        total_score = sum(c.score for c in checks) / len(checks) if checks else 0
        passed_count = sum(1 for c in checks if c.passed)
        failed_count = len(checks) - passed_count

        logger.debug(f"SEO 检查完成: score={total_score:.1f}, passed={passed_count}/{len(checks)}")

        return SEOReport(
            url=url,
            total_score=total_score,
            checks=checks,
            passed_count=passed_count,
            failed_count=failed_count,
        )

    async def _check_title(self) -> SEOCheckResult:
        """检查页面标题"""
        title = await self.page.title()
        length = len(title)
        passed = 10 <= length <= 60

        if length == 0:
            description = "缺少页面标题"
            score = 0
        elif length < 10:
            description = f"标题过短 ({length} 字符)，建议 10-60 字符"
            score = 50
        elif length > 60:
            description = f"标题过长 ({length} 字符)，建议 10-60 字符"
            score = 50
        else:
            description = f"标题长度适中 ({length} 字符)"
            score = 100

        return SEOCheckResult(
            id="title",
            name="页面标题",
            passed=passed,
            score=score,
            description=description,
            details=[title] if title else [],
        )

    async def _check_meta_description(self) -> SEOCheckResult:
        """检查 Meta Description"""
        description_el = self.page.locator('meta[name="description"]')
        content = await description_el.get_attribute("content")
        length = len(content) if content else 0
        passed = 50 <= length <= 160

        if length == 0:
            description = "缺少 Meta Description"
            score = 0
        elif length < 50:
            description = f"Meta Description 过短 ({length} 字符)，建议 50-160 字符"
            score = 50
        elif length > 160:
            description = f"Meta Description 过长 ({length} 字符)，建议 50-160 字符"
            score = 50
        else:
            description = f"Meta Description 长度适中 ({length} 字符)"
            score = 100

        return SEOCheckResult(
            id="meta-description",
            name="Meta Description",
            passed=passed,
            score=score,
            description=description,
            details=[content[:100]] if content else [],
        )

    async def _check_h1(self) -> SEOCheckResult:
        """检查 H1 标题"""
        h1_count = await self.page.locator("h1").count()
        passed = h1_count == 1

        if h1_count == 0:
            description = "缺少 H1 标题"
            score = 0
        elif h1_count == 1:
            description = "存在唯一的 H1 标题"
            score = 100
        else:
            description = f"存在 {h1_count} 个 H1 标题，建议只保留一个"
            score = 50

        return SEOCheckResult(
            id="h1",
            name="H1 标题",
            passed=passed,
            score=score,
            description=description,
        )

    async def _check_images_alt(self) -> SEOCheckResult:
        """检查图片 Alt 属性"""
        images = self.page.locator("img")
        total = await images.count()

        if total == 0:
            return SEOCheckResult(
                id="images-alt",
                name="图片 Alt 属性",
                passed=True,
                score=100,
                description="页面没有图片",
            )

        with_alt = 0
        for i in range(total):
            alt = await images.nth(i).get_attribute("alt")
            if alt is not None:
                with_alt += 1

        score = (with_alt / total) * 100
        passed = with_alt == total

        return SEOCheckResult(
            id="images-alt",
            name="图片 Alt 属性",
            passed=passed,
            score=score,
            description=f"{with_alt}/{total} 张图片有 Alt 属性",
            details=[f"{total - with_alt} 张图片缺少 Alt 属性"] if not passed else [],
        )

    async def _check_canonical(self) -> SEOCheckResult:
        """检查 Canonical URL"""
        canonical = await self.page.locator('link[rel="canonical"]').get_attribute("href")
        passed = bool(canonical)

        return SEOCheckResult(
            id="canonical",
            name="Canonical URL",
            passed=passed,
            score=100 if passed else 0,
            description="存在 Canonical URL" if passed else "缺少 Canonical URL",
            details=[canonical] if canonical else [],
        )

    async def _check_viewport(self) -> SEOCheckResult:
        """检查 Viewport 设置"""
        viewport = await self.page.locator('meta[name="viewport"]').get_attribute("content")
        passed = bool(viewport and "width=device-width" in viewport)

        return SEOCheckResult(
            id="meta-viewport",
            name="Viewport 设置",
            passed=passed,
            score=100 if passed else 0,
            description="Viewport 设置正确" if passed else "缺少正确的 Viewport 设置",
            details=[viewport] if viewport else [],
        )

    async def _check_robots(self) -> SEOCheckResult:
        """检查 Robots Meta"""
        robots = await self.page.locator('meta[name="robots"]').get_attribute("content")

        if robots and "noindex" in robots.lower():
            return SEOCheckResult(
                id="robots",
                name="Robots Meta",
                passed=False,
                score=0,
                description="页面被设置为 noindex",
                details=[robots],
            )

        return SEOCheckResult(
            id="robots",
            name="Robots Meta",
            passed=True,
            score=100,
            description="页面可被索引" if not robots else f"Robots: {robots}",
            details=[robots] if robots else [],
        )

    async def _check_og_tags(self) -> SEOCheckResult:
        """检查 Open Graph 标签"""
        og_title = await self.page.locator('meta[property="og:title"]').get_attribute("content")
        og_description = await self.page.locator('meta[property="og:description"]').get_attribute("content")
        og_image = await self.page.locator('meta[property="og:image"]').get_attribute("content")

        tags = []
        if og_title:
            tags.append("og:title")
        if og_description:
            tags.append("og:description")
        if og_image:
            tags.append("og:image")

        score = (len(tags) / 3) * 100
        passed = len(tags) >= 2

        return SEOCheckResult(
            id="og-tags",
            name="Open Graph 标签",
            passed=passed,
            score=score,
            description=f"存在 {len(tags)}/3 个主要 OG 标签",
            details=tags,
        )

    async def _check_structured_data(self) -> SEOCheckResult:
        """检查结构化数据"""
        json_ld = await self.page.locator('script[type="application/ld+json"]').count()

        passed = json_ld > 0
        return SEOCheckResult(
            id="structured-data",
            name="结构化数据",
            passed=passed,
            score=100 if passed else 0,
            description=f"存在 {json_ld} 个结构化数据块" if passed else "缺少结构化数据",
        )

    async def _check_links(self) -> SEOCheckResult:
        """检查链接"""
        links = self.page.locator("a[href]")
        total = await links.count()

        broken_count = 0
        for i in range(min(total, 20)):  # 只检查前20个
            link = links.nth(i)
            href = await link.get_attribute("href")
            if href and href.startswith("javascript:"):
                broken_count += 1

        score = 100 if broken_count == 0 else max(0, 100 - broken_count * 10)
        passed = broken_count == 0

        return SEOCheckResult(
            id="links",
            name="链接检查",
            passed=passed,
            score=score,
            description=f"检查了 {min(total, 20)} 个链接",
            details=[f"{broken_count} 个问题链接"] if broken_count else [],
        )

    def generate_report(self, report: SEOReport) -> str:
        """生成报告"""
        lines = [
            "\n=== SEO 检查报告 ===\n",
            f"URL: {report.url}",
            f"总分: {report.total_score:.1f}/100",
            f"通过: {report.passed_count}/{len(report.checks)}\n",
        ]

        for check in report.checks:
            status = "✅" if check.passed else "❌"
            lines.append(f"{status} {check.name}: {check.description} ({check.score:.0f}分)")
            if check.details:
                for detail in check.details[:3]:
                    lines.append(f"   - {detail}")

        return "\n".join(lines)


def create_seo_checker(page: Page) -> SEOChecker:
    """创建 SEO 检查器实例"""
    return SEOChecker(page)



"""
标签页管理 Mixin
"""
from typing import Optional, TYPE_CHECKING

from loguru import logger

if TYPE_CHECKING:
    from ..tab import Tab


class TabManagerMixin:
    """标签页管理功能"""

    # 这些属性由 BrowserCore 定义，这里声明类型供类型检查
    _tabs: dict[str, "Tab"]
    _current_tab: Optional[str]
    _tab_counter: int
    _context: "BrowserContext"
    config: "Config"

    @property
    def page(self) -> "Page":
        """获取当前标签页的 Page 对象"""
        return self.get_tab().page

    async def new_tab(self, name: Optional[str] = None) -> "Tab":
        """
        创建新标签页

        Args:
            name: 标签页名称

        Returns:
            Tab: 新创建的标签页
        """
        from ..tab import Tab
        from ...anti_detection import AntiDetection

        if len(self._tabs) >= self.config.max_tabs:
            raise ValueError(f"已达到最大标签页数量: {self.config.max_tabs}")

        self._tab_counter += 1
        tab_id = f"tab_{self._tab_counter}"
        name = name or tab_id

        page = await self._context.new_page()

        # 应用页面隐身处理
        if self.config.stealth_enabled:
            await AntiDetection.stealth_page(page)

        tab = Tab(id=tab_id, page=page, name=name)

        self._tabs[tab_id] = tab

        if self._current_tab is None:
            self._current_tab = tab_id

        logger.info(f"创建新标签页: id={tab_id}, name={name}")
        return tab

    async def close_tab(self, tab_id: Optional[str] = None):
        """
        关闭标签页

        Args:
            tab_id: 标签页ID，默认关闭当前标签页
        """
        tab_id = tab_id or self._current_tab
        if tab_id is None:
            return

        tab = self._tabs.get(tab_id)
        if tab:
            await tab.page.close()
            del self._tabs[tab_id]
            logger.info(f"关闭标签页: id={tab_id}")

            if self._current_tab == tab_id:
                self._current_tab = next(iter(self._tabs), None) if self._tabs else None

    def switch_tab(self, tab_id: str) -> "Tab":
        """
        切换到指定标签页

        Args:
            tab_id: 标签页ID

        Returns:
            Tab: 目标标签页
        """
        if tab_id not in self._tabs:
            raise ValueError(f"标签页不存在: {tab_id}")

        self._current_tab = tab_id
        tab = self._tabs[tab_id]
        logger.info(f"切换到标签页: id={tab_id}, name={tab.name}")
        return tab

    def get_tab(self, tab_id: Optional[str] = None) -> "Tab":
        """
        获取标签页

        Args:
            tab_id: 标签页ID，默认返回当前标签页

        Returns:
            Tab: 标签页
        """
        tab_id = tab_id or self._current_tab
        if tab_id is None:
            raise ValueError("没有可用的标签页")
        return self._tabs[tab_id]

    def list_tabs(self) -> list["Tab"]:
        """列出所有标签页"""
        return list(self._tabs.values())



"""
配置预设模块
提供常用场景的预设配置
"""
from typing import Dict, Any, Optional
from dataclasses import dataclass, field
from playwright_ez.config import Config


@dataclass
class PresetConfig:
    """预设配置"""
    name: str
    description: str
    config: Dict[str, Any] = field(default_factory=dict)


# 预设配置集合
PRESETS: Dict[str, PresetConfig] = {
    # 基础预设
    "default": PresetConfig(
        name="default",
        description="默认配置：无头模式，基本功能",
        config={
            "headless": True,
            "highlight_enabled": True,
            "log_level": "INFO",
        }
    ),
    
    # 调试模式
    "debug": PresetConfig(
        name="debug",
        description="调试模式：显示浏览器，详细日志，元素高亮",
        config={
            "headless": False,
            "highlight_enabled": True,
            "highlight_color": "red",
            "highlight_duration": 1.0,
            "log_level": "DEBUG",
            "default_timeout": 60.0,
        }
    ),
    
    # 快速模式
    "fast": PresetConfig(
        name="fast",
        description="快速模式：缩短等待时间，加快执行",
        config={
            "headless": True,
            "highlight_enabled": False,
            "log_level": "WARNING",
            "default_timeout": 10.0,
            "poll_interval": 0.05,
        }
    ),
    
    # 稳定模式
    "stable": PresetConfig(
        name="stable",
        description="稳定模式：增加等待时间，提高稳定性",
        config={
            "headless": True,
            "highlight_enabled": False,
            "log_level": "INFO",
            "default_timeout": 60.0,
            "poll_interval": 0.2,
            "stability_threshold": 5,
        }
    ),
    
    # 反检测模式
    "stealth": PresetConfig(
        name="stealth",
        description="隐身模式：隐藏自动化特征",
        config={
            "headless": True,
            "highlight_enabled": False,
            "log_level": "WARNING",
        }
    ),
    
    # 视觉测试模式
    "visual": PresetConfig(
        name="visual",
        description="视觉测试模式：启用截图和视觉对比",
        config={
            "headless": True,
            "highlight_enabled": False,
            "log_level": "INFO",
            "screenshot_on_error": True,
        }
    ),
    
    # 性能测试模式
    "performance": PresetConfig(
        name="performance",
        description="性能测试模式：记录性能指标",
        config={
            "headless": True,
            "highlight_enabled": False,
            "log_level": "INFO",
        }
    ),
    
    # 移动端模式
    "mobile": PresetConfig(
        name="mobile",
        description="移动端模式：模拟移动设备",
        config={
            "headless": True,
            "highlight_enabled": True,
            "log_level": "INFO",
        }
    ),
    
    # CI/CD模式
    "ci": PresetConfig(
        name="ci",
        description="CI/CD模式：无头，静默，快速失败",
        config={
            "headless": True,
            "highlight_enabled": False,
            "log_level": "ERROR",
            "default_timeout": 30.0,
        }
    ),
    
    # 开发模式
    "dev": PresetConfig(
        name="dev",
        description="开发模式：显示浏览器，详细日志，慢动作",
        config={
            "headless": False,
            "highlight_enabled": True,
            "highlight_duration": 1.5,
            "log_level": "DEBUG",
            "default_timeout": 120.0,
        }
    ),
}


def get_preset(name: str) -> Optional[Config]:
    """
    获取预设配置
    
    Args:
        name: 预设名称
        
    Returns:
        Config: 配置对象
    """
    if name not in PRESETS:
        raise ValueError(f"预设不存在: {name}. 可用: {list(PRESETS.keys())}")
    
    preset = PRESETS[name]
    return Config(**preset.config)


def list_presets() -> Dict[str, str]:
    """列出所有预设"""
    return {name: preset.description for name, preset in PRESETS.items()}


def create_custom_preset(name: str, description: str, **kwargs) -> PresetConfig:
    """
    创建自定义预设
    
    Args:
        name: 预设名称
        description: 描述
        **kwargs: 配置参数
        
    Returns:
        PresetConfig: 预设配置
    """
    preset = PresetConfig(
        name=name,
        description=description,
        config=kwargs,
    )
    
    PRESETS[name] = preset
    return preset


# 常用场景预设
SCENARIO_PRESETS = {
    # 登录场景
    "login": {
        "wait_after_fill": 0.5,
        "wait_after_click": 1.0,
        "timeout": 30,
    },
    
    # 表单场景
    "form": {
        "wait_after_fill": 0.3,
        "validate_before_submit": True,
    },
    
    # 数据抓取场景
    "scraping": {
        "wait_for_load": True,
        "scroll_to_load": True,
        "retry_count": 3,
    },
    
    # 电商场景
    "ecommerce": {
        "add_to_cart_wait": 2.0,
        "checkout_timeout": 60,
    },
}


def get_scenario_config(scenario: str) -> Dict[str, Any]:
    """获取场景配置"""
    return SCENARIO_PRESETS.get(scenario, {})


# 浏览器启动参数预设
BROWSER_ARGS = {
    "fast": [
        "--disable-extensions",
        "--disable-images",
        "--disable-javascript",
    ],
    
    "stealth": [
        "--disable-blink-features=AutomationControlled",
        "--disable-extensions",
        "--disable-infobars",
    ],
    
    "memory_saving": [
        "--disable-extensions",
        "--disable-plugins",
        "--disable-images",
        "--disable-background-networking",
    ],
    
    "full_features": [
        "--enable-features=NetworkService,NetworkServiceInProcess",
    ],
}


def get_browser_args(preset: str) -> list:
    """获取浏览器启动参数"""
    return BROWSER_ARGS.get(preset, [])



"""
混合引擎
结合 LLM 和规则引擎，提供智能回退机制
"""
import json
from typing import Optional, List

from playwright.async_api import Page
from loguru import logger

from .base import BaseEngine
from .rule_engine import RuleEngine
from ..llm.factory import create_llm_provider
from ..llm.base import BaseLLMProvider, LLMResponse
from ..types import (
    AIConfig,
    SelectorSuggestion,
    ErrorDiagnosis,
    CodeSuggestion,
    LLMProvider as ProviderEnum,
)


class HybridEngine(BaseEngine):
    """
    混合引擎：LLM + 规则引擎

    工作流程:
    1. 如果配置为 RULE 提供商，直接使用规则引擎
    2. 如果 use_llm_first=True，优先尝试 LLM
    3. 如果 LLM 失败且 fallback_to_rule=True，回退到规则引擎
    """

    def __init__(self, page: Page, config: Optional[AIConfig] = None):
        self.page = page
        self.config = config or AIConfig()
        self._rule_engine = RuleEngine(page, self.config)
        self._llm: Optional[BaseLLMProvider] = None

    @property
    def llm(self) -> Optional[BaseLLMProvider]:
        """获取 LLM 提供商（延迟初始化）"""
        if self._llm is None and self.config.llm.provider != ProviderEnum.RULE:
            self._llm = create_llm_provider(self.config.llm)
        return self._llm

    async def suggest_selectors(
        self,
        description: str,
        element_type: Optional[str] = None,
    ) -> List[SelectorSuggestion]:
        """
        智能选择器建议

        优先使用 LLM，失败时回退到规则引擎
        """
        # 尝试 LLM
        if self.config.use_llm_first and self.llm:
            suggestions = await self._llm_suggest_selectors(description, element_type)
            if suggestions:
                logger.debug(f"LLM 生成了 {len(suggestions)} 个选择器建议")
                return suggestions

        # 回退到规则引擎
        if self.config.fallback_to_rule or not self.llm:
            return await self._rule_engine.suggest_selectors(description, element_type)

        return []

    async def _llm_suggest_selectors(
        self,
        description: str,
        element_type: Optional[str],
    ) -> Optional[List[SelectorSuggestion]]:
        """使用 LLM 生成选择器建议"""
        prompt = f"""分析以下元素描述，生成最合适的 CSS 选择器。

描述: {description}
元素类型: {element_type or "未知"}

请以 JSON 格式返回选择器建议列表:
[
  {{"selector": "...", "confidence": 0.9, "strategy": "..."}}
]

要求:
1. 只返回 JSON 数组，不要其他内容
2. confidence 是 0-1 之间的浮点数
3. strategy 描述选择器策略（如 "id", "class", "text" 等）
4. 最多返回 5 个建议"""

        response = await self.llm.call(prompt, system_prompt=self._get_system_prompt())

        if not response.success:
            logger.warning(f"LLM 调用失败: {response.error}")
            return None

        # 解析 JSON 响应
        try:
            # 尝试提取 JSON 数组
            content = response.content.strip()
            # 处理可能的 markdown 代码块
            if content.startswith("```"):
                lines = content.split("\n")
                content = "\n".join(lines[1:-1] if lines[-1] == "```" else lines[1:])

            data = json.loads(content)
            if not isinstance(data, list):
                data = [data]

            suggestions = []
            for item in data[:self.config.max_suggestions]:
                selector = item.get("selector", "")
                if not selector:
                    continue

                # 验证选择器
                confidence = await self._validate_selector(selector)
                if confidence > 0:
                    suggestions.append(SelectorSuggestion(
                        selector=selector,
                        confidence=min(confidence, item.get("confidence", 0.8)),
                        strategy=item.get("strategy", "llm"),
                        element_type=element_type or "unknown",
                        description="由 LLM 生成",
                    ))

            return suggestions if suggestions else None

        except json.JSONDecodeError as e:
            logger.warning(f"LLM 返回的 JSON 解析失败: {e}")
            return None
        except Exception as e:
            logger.warning(f"处理 LLM 响应时出错: {e}")
            return None

    async def _validate_selector(self, selector: str) -> float:
        """验证选择器并返回置信度"""
        try:
            locator = self.page.locator(selector)
            count = await locator.count()

            if count == 0:
                return 0.0
            elif count == 1:
                return 1.0
            else:
                return max(0.5, 1.0 - (count - 1) * 0.1)

        except Exception:
            return 0.0

    async def diagnose_error(
        self,
        error: Exception,
        context: Optional[dict] = None,
    ) -> ErrorDiagnosis:
        """诊断错误"""
        # 尝试 LLM 诊断
        if self.config.use_llm_first and self.llm:
            diagnosis = await self._llm_diagnose_error(error, context)
            if diagnosis:
                logger.debug("LLM 完成了错误诊断")
                return diagnosis

        # 回退到规则引擎
        return await self._rule_engine.diagnose_error(error, context)

    async def _llm_diagnose_error(
        self,
        error: Exception,
        context: Optional[dict],
    ) -> Optional[ErrorDiagnosis]:
        """使用 LLM 诊断错误"""
        error_message = str(error)
        context_str = f"\n上下文: {context}" if context else ""

        prompt = f"""分析以下自动化测试错误，诊断原因并提供解决方案。

错误信息: {error_message}{context_str}

请以 JSON 格式返回诊断结果:
{{
  "error_type": "错误类型",
  "likely_causes": ["原因1", "原因2"],
  "suggested_solutions": ["解决方案1", "解决方案2"],
  "related_elements": ["相关元素选择器"]
}}

只返回 JSON，不要其他内容。"""

        response = await self.llm.call(prompt, system_prompt=self._get_system_prompt())

        if not response.success:
            return None

        try:
            content = response.content.strip()
            if content.startswith("```"):
                lines = content.split("\n")
                content = "\n".join(lines[1:-1] if lines[-1] == "```" else lines[1:])

            data = json.loads(content)

            return ErrorDiagnosis(
                error_type=data.get("error_type", "unknown"),
                likely_causes=data.get("likely_causes", []),
                suggested_solutions=data.get("suggested_solutions", []),
                related_elements=data.get("related_elements", []),
            )
        except Exception as e:
            logger.warning(f"解析 LLM 错误诊断结果失败: {e}")
            return None

    async def generate_test_code(
        self,
        scenario: str,
        include_assertions: bool = True,
    ) -> CodeSuggestion:
        """生成测试代码"""
        # 尝试 LLM 生成
        if self.config.use_llm_first and self.llm:
            code = await self._llm_generate_test_code(scenario, include_assertions)
            if code:
                logger.debug("LLM 生成了测试代码")
                return code

        # 回退到规则引擎
        return await self._rule_engine.generate_test_code(scenario, include_assertions)

    async def _llm_generate_test_code(
        self,
        scenario: str,
        include_assertions: bool,
    ) -> Optional[CodeSuggestion]:
        """使用 LLM 生成测试代码"""
        assertions_str = "包含断言" if include_assertions else "不包含断言"

        prompt = f"""根据以下场景生成 Python Playwright 自动化测试代码。

场景: {scenario}
要求: {assertions_str}

使用 playwright-ez 库的 EasyBrowser 类:
```python
async with EasyBrowser() as browser:
    await browser.goto("url")
    await browser.click("selector")
```

只返回代码，不要解释。"""

        response = await self.llm.call(prompt, system_prompt=self._get_system_prompt())

        if not response.success:
            return None

        try:
            code = response.content.strip()
            # 移除可能的 markdown 代码块标记
            if code.startswith("```python"):
                lines = code.split("\n")
                code = "\n".join(lines[1:-1] if lines[-1] == "```" else lines[1:])
            elif code.startswith("```"):
                lines = code.split("\n")
                code = "\n".join(lines[1:-1] if lines[-1] == "```" else lines[1:])

            return CodeSuggestion(
                code=code,
                description=f"根据场景 '{scenario}' 生成的测试代码",
                language="python",
                confidence=0.8,
            )
        except Exception as e:
            logger.warning(f"处理 LLM 生成的代码失败: {e}")
            return None

    def _get_system_prompt(self) -> str:
        """获取系统提示"""
        return """你是一个 Web 自动化测试专家，精通 Playwright 和 CSS 选择器。
你的任务是帮助用户：
1. 分析网页元素，生成精确的 CSS 选择器
2. 诊断自动化测试中的错误，提供解决方案
3. 根据测试场景生成自动化测试代码

请始终返回有效的 JSON 格式数据。"""



"""
自动录制器
支持操作录制、代码回放
"""
import json
import asyncio
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any
from datetime import datetime
from pathlib import Path
from playwright.async_api import Page
from loguru import logger


@dataclass
class AutoRecorderConfig:
    """自动录制配置"""
    record_clicks: bool = True
    record_input: bool = True
    record_navigation: bool = True
    record_scrolls: bool = False
    record_mouse_movement: bool = False
    record_keypress: bool = True
    ignore_selectors: List[str] = field(default_factory=list)
    max_actions: int = 1000


@dataclass
class RecordedAction:
    """录制的动作"""
    type: str  # click, input, navigate, scroll, keypress
    timestamp: str
    selector: Optional[str] = None
    value: Optional[str] = None
    tag: Optional[str] = None
    text: Optional[str] = None
    url: Optional[str] = None
    x: Optional[int] = None
    y: Optional[int] = None
    key: Optional[str] = None
    modifiers: List[str] = field(default_factory=list)
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class RecordingSession:
    """录制会话"""
    id: str
    start_time: str
    end_time: Optional[str] = None
    url: Optional[str] = None
    actions: List[RecordedAction] = field(default_factory=list)

    @property
    def action_count(self) -> int:
        return len(self.actions)

    @property
    def duration_seconds(self) -> float:
        if not self.end_time:
            return 0.0
        start = datetime.fromisoformat(self.start_time)
        end = datetime.fromisoformat(self.end_time)
        return (end - start).total_seconds()


class AutoRecorder:
    """
    自动录制器
    录制用户操作并生成代码
    """

    def __init__(
        self,
        page: Page,
        config: Optional[AutoRecorderConfig] = None,
    ):
        self.page = page
        self.config = config or AutoRecorderConfig()
        self._session: Optional[RecordingSession] = None
        self._is_recording = False

    # ==================== 录制控制 ====================

    async def start(self, url: Optional[str] = None):
        """开始录制"""
        if self._is_recording:
            logger.warning("录制已在进行中")
            return

        import uuid
        self._session = RecordingSession(
            id=str(uuid.uuid4())[:8],
            start_time=datetime.now().isoformat(),
            url=url or self.page.url,
        )
        self._is_recording = True
        await self._inject_recorder()
        await self._setup_listeners()
        logger.info(f"开始录制: {self._session.id}")

    async def stop(self) -> RecordingSession:
        """停止录制"""
        if not self._is_recording or not self._session:
            raise RuntimeError("没有正在进行的录制")

        self._is_recording = False
        self._session.end_time = datetime.now().isoformat()
        await self._remove_listeners()
        logger.info(f"录制结束: {self._session.id}, 动作数: {self._session.action_count}")
        return self._session

    async def pause(self):
        """暂停录制"""
        self._is_recording = False

    async def resume(self):
        """恢复录制"""
        self._is_recording = True

    # ==================== 内部方法 ====================

    async def _inject_recorder(self):
        """注入录制脚本"""
        ignore_selectors = json.dumps(self.config.ignore_selectors)
        await self.page.evaluate(f"""
            window.__recorderActions = window.__recorderActions || [];
            window.__recorderIgnore = {ignore_selectors};

            function getSelector(element) {{
                if (element.id) return '#' + element.id;
                if (element.dataset && element.dataset.testid) {{
                    return '[data-testid="' + element.dataset.testid + '"]';
                }}
                let selector = element.tagName.toLowerCase();
                if (element.className && typeof element.className === 'string') {{
                    const classes = element.className.split(' ').filter(c => c && !c.includes(':'));
                    if (classes.length) selector += '.' + classes.slice(0, 2).join('.');
                }}
                return selector;
            }}

            function shouldIgnore(element) {{
                const ignore = window.__recorderIgnore || [];
                for (const sel of ignore) {{
                    if (element.matches && element.matches(sel)) return true;
                }}
                return false;
            }}

            function recordAction(action) {{
                action.timestamp = new Date().toISOString();
                window.__recorderActions.push(action);
                if (window.__recorderActions.length > {self.config.max_actions}) {{
                    window.__recorderActions.shift();
                }}
            }}

            window.__recorderGetSelector = getSelector;
            window.__recorderRecord = recordAction;
            window.__recorderShouldIgnore = shouldIgnore;
        """)

    async def _setup_listeners(self):
        """设置事件监听"""
        if self.config.record_clicks:
            await self.page.evaluate("""
                window.__clickHandler = (e) => {
                    if (!window.__recordingActive) return;
                    if (window.__recorderShouldIgnore(e.target)) return;
                    window.__recorderRecord({
                        type: 'click',
                        selector: window.__recorderGetSelector(e.target),
                        tag: e.target.tagName.toLowerCase(),
                        text: e.target.innerText?.substring(0, 50),
                        x: e.clientX,
                        y: e.clientY,
                    });
                };
                document.addEventListener('click', window.__clickHandler, true);
            """)

        if self.config.record_input:
            await self.page.evaluate("""
                window.__inputHandler = (e) => {
                    if (!window.__recordingActive) return;
                    if (window.__recorderShouldIgnore(e.target)) return;
                    const element = e.target;
                    const type = element.type || 'text';
                    const value = type === 'password' ? '***' : element.value;
                    window.__recorderRecord({
                        type: 'input',
                        selector: window.__recorderGetSelector(element),
                        value: value,
                        tag: element.tagName.toLowerCase(),
                        inputType: type,
                    });
                };
                document.addEventListener('input', window.__inputHandler, true);
            """)

        if self.config.record_navigation:
            self.page.on("framenavigated", self._on_navigation)

        await self.page.evaluate("window.__recordingActive = true;")

    async def _remove_listeners(self):
        """移除事件监听"""
        try:
            await self.page.evaluate("""
                window.__recordingActive = false;
                if (window.__clickHandler) {
                    document.removeEventListener('click', window.__clickHandler, true);
                }
                if (window.__inputHandler) {
                    document.removeEventListener('input', window.__inputHandler, true);
                }
            """)
            self.page.remove_listener("framenavigated", self._on_navigation)
        except Exception as e:
            logger.warning(f"移除监听器失败: {e}")

    def _on_navigation(self, frame):
        """导航事件处理"""
        if not self._is_recording or not self._session:
            return
        action = RecordedAction(
            type="navigate",
            timestamp=datetime.now().isoformat(),
            url=frame.url,
        )
        self._session.actions.append(action)

    # ==================== 导出功能 ====================

    async def collect_actions(self) -> List[RecordedAction]:
        """收集录制的动作"""
        if not self._session:
            return []
        actions_data = await self.page.evaluate("window.__recorderActions || []")
        for action_data in actions_data:
            action = RecordedAction(
                type=action_data.get("type", "unknown"),
                timestamp=action_data.get("timestamp", datetime.now().isoformat()),
                selector=action_data.get("selector"),
                value=action_data.get("value"),
                tag=action_data.get("tag"),
                text=action_data.get("text"),
            )
            exists = any(
                a.timestamp == action.timestamp and a.type == action.type
                for a in self._session.actions
            )
            if not exists:
                self._session.actions.append(action)
        return self._session.actions

    async def export_json(self, file_path: str):
        """导出为 JSON"""
        await self.collect_actions()
        if not self._session:
            raise RuntimeError("没有录制会话")

        data = {
            "id": self._session.id,
            "start_time": self._session.start_time,
            "end_time": self._session.end_time,
            "url": self._session.url,
            "actions": [
                {"type": a.type, "timestamp": a.timestamp, "selector": a.selector, "value": a.value}
                for a in self._session.actions
            ],
        }
        Path(file_path).parent.mkdir(parents=True, exist_ok=True)
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
        logger.info(f"录制已导出: {file_path}")

    async def export_python_code(self, file_path: str):
        """导出为 Python 代码"""
        await self.collect_actions()
        if not self._session:
            raise RuntimeError("没有录制会话")

        lines = [
            '"""自动生成的测试代码"""',
            'import asyncio',
            'from playwright_ez import EasyBrowser',
            '',
            'async def recorded_test():',
            '    async with EasyBrowser() as browser:',
        ]
        for action in self._session.actions:
            if action.type == "navigate":
                lines.append(f'        await browser.goto("{action.url}")')
            elif action.type == "click" and action.selector:
                lines.append(f'        await browser.click("{action.selector}")')
            elif action.type == "input" and action.selector and action.value:
                escaped = action.value.replace('"', '\\"')
                lines.append(f'        await browser.type_text("{action.selector}", "{escaped}")')

        lines.extend(['', 'if __name__ == "__main__":', '    asyncio.run(recorded_test())'])

        Path(file_path).parent.mkdir(parents=True, exist_ok=True)
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write('\n'.join(lines))
        logger.info(f"Python 代码已导出: {file_path}")


def create_auto_recorder(page: Page, config: Optional[AutoRecorderConfig] = None) -> AutoRecorder:
    """创建自动录制器实例"""
    return AutoRecorder(page, config)



"""
工具函数模块
常用工具函数集合
"""
import re
import json
import random
import string
import hashlib
import base64
from typing import Optional, List, Dict, Any, Union, Tuple
from datetime import datetime, timedelta
from pathlib import Path
from loguru import logger


# ==================== 字符串工具 ====================

def random_string(length: int = 10, chars: str = None) -> str:
    """
    生成随机字符串
    
    Args:
        length: 长度
        chars: 字符集，默认字母数字
        
    Returns:
        str: 随机字符串
    """
    if chars is None:
        chars = string.ascii_letters + string.digits
    return ''.join(random.choice(chars) for _ in range(length))


def random_email(domain: str = "test.com") -> str:
    """生成随机邮箱"""
    return f"{random_string(8).lower()}@{domain}"


def random_phone(country_code: str = "86") -> str:
    """生成随机手机号"""
    if country_code == "86":
        # 中国手机号
        prefixes = ["130", "131", "132", "133", "135", "136", "137", "138", "139",
                   "150", "151", "152", "153", "155", "156", "157", "158", "159",
                   "180", "181", "182", "183", "184", "185", "186", "187", "188", "189"]
        return random.choice(prefixes) + ''.join(random.choice(string.digits) for _ in range(8))
    else:
        return '+' + country_code + ''.join(random.choice(string.digits) for _ in range(10))


def random_password(length: int = 12, include_special: bool = True) -> str:
    """生成随机密码"""
    chars = string.ascii_letters + string.digits
    if include_special:
        chars += "!@#$%^&*"
    
    # 确保包含各种类型字符
    password = [
        random.choice(string.ascii_uppercase),
        random.choice(string.ascii_lowercase),
        random.choice(string.digits),
    ]
    
    if include_special:
        password.append(random.choice("!@#$%^&*"))
    
    # 填充剩余长度
    password += [random.choice(chars) for _ in range(length - len(password))]
    random.shuffle(password)
    
    return ''.join(password)


def mask_sensitive(text: str, show_len: int = 4, mask_char: str = "*") -> str:
    """
    敏感信息脱敏
    
    Args:
        text: 原文
        show_len: 显示长度
        mask_char: 掩码字符
        
    Returns:
        str: 脱敏后的文本
    """
    if len(text) <= show_len * 2:
        return mask_char * len(text)
    
    return text[:show_len] + mask_char * (len(text) - show_len * 2) + text[-show_len:]


def truncate(text: str, max_len: int = 100, suffix: str = "...") -> str:
    """截断文本"""
    if len(text) <= max_len:
        return text
    return text[:max_len - len(suffix)] + suffix


def extract_numbers(text: str) -> List[str]:
    """提取文本中的数字"""
    return re.findall(r'\d+\.?\d*', text)


def extract_urls(text: str) -> List[str]:
    """提取文本中的URL"""
    pattern = r'https?://[^\s<>"{}|\\^`\[\]]+'
    return re.findall(pattern, text)


def extract_emails(text: str) -> List[str]:
    """提取文本中的邮箱"""
    pattern = r'[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}'
    return re.findall(pattern, text)


def extract_phones(text: str) -> List[str]:
    """提取文本中的电话号码"""
    pattern = r'1[3-9]\d{9}'
    return re.findall(pattern, text)


def to_snake_case(text: str) -> str:
    """转换为蛇形命名"""
    s1 = re.sub(r'(.)([A-Z][a-z]+)', r'\1_\2', text)
    return re.sub(r'([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def to_camel_case(text: str) -> str:
    """转换为驼峰命名"""
    components = text.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])


def to_kebab_case(text: str) -> str:
    """转换为短横线命名"""
    return to_snake_case(text).replace('_', '-')


# ==================== 编码工具 ====================

def base64_encode(text: str) -> str:
    """Base64编码"""
    return base64.b64encode(text.encode()).decode()


def base64_decode(encoded: str) -> str:
    """Base64解码"""
    return base64.b64decode(encoded.encode()).decode()


def md5_hash(text: str) -> str:
    """MD5哈希"""
    return hashlib.md5(text.encode()).hexdigest()


def sha256_hash(text: str) -> str:
    """SHA256哈希"""
    return hashlib.sha256(text.encode()).hexdigest()


def url_encode(text: str) -> str:
    """URL编码"""
    from urllib.parse import quote
    return quote(text)


def url_decode(encoded: str) -> str:
    """URL解码"""
    from urllib.parse import unquote
    return unquote(encoded)


# ==================== 时间工具 ====================

def now_str(format: str = "%Y-%m-%d %H:%M:%S") -> str:
    """当前时间字符串"""
    return datetime.now().strftime(format)


def timestamp() -> int:
    """当前时间戳（秒）"""
    return int(datetime.now().timestamp())


def timestamp_ms() -> int:
    """当前时间戳（毫秒）"""
    return int(datetime.now().timestamp() * 1000)


def parse_datetime(text: str, format: str = None) -> datetime:
    """解析日期时间"""
    if format:
        return datetime.strptime(text, format)
    
    # 尝试多种格式
    formats = [
        "%Y-%m-%d %H:%M:%S",
        "%Y-%m-%d %H:%M",
        "%Y-%m-%d",
        "%Y/%m/%d %H:%M:%S",
        "%Y/%m/%d",
        "%d/%m/%Y",
    ]
    
    for fmt in formats:
        try:
            return datetime.strptime(text, fmt)
        except ValueError:
            continue
    
    raise ValueError(f"无法解析日期时间: {text}")


def time_ago(dt: datetime) -> str:
    """人性化时间显示"""
    now = datetime.now()
    diff = now - dt
    
    if diff.seconds < 60:
        return "刚刚"
    elif diff.seconds < 3600:
        return f"{diff.seconds // 60}分钟前"
    elif diff.seconds < 86400:
        return f"{diff.seconds // 3600}小时前"
    elif diff.days < 30:
        return f"{diff.days}天前"
    elif diff.days < 365:
        return f"{diff.days // 30}个月前"
    else:
        return f"{diff.days // 365}年前"


# ==================== 文件工具 ====================

def ensure_dir(path: str):
    """确保目录存在"""
    Path(path).mkdir(parents=True, exist_ok=True)


def read_file(path: str, encoding: str = "utf-8") -> str:
    """读取文件"""
    return Path(path).read_text(encoding=encoding)


def write_file(path: str, content: str, encoding: str = "utf-8"):
    """写入文件"""
    Path(path).parent.mkdir(parents=True, exist_ok=True)
    Path(path).write_text(content, encoding=encoding)


def read_json(path: str) -> Any:
    """读取JSON文件"""
    with open(path, 'r', encoding='utf-8') as f:
        return json.load(f)


def write_json(path: str, data: Any, indent: int = 2):
    """写入JSON文件"""
    Path(path).parent.mkdir(parents=True, exist_ok=True)
    with open(path, 'w', encoding='utf-8') as f:
        json.dump(data, f, ensure_ascii=False, indent=indent)


def file_size(path: str) -> int:
    """文件大小（字节）"""
    return Path(path).stat().st_size


def file_size_human(path: str) -> str:
    """人性化文件大小"""
    size = file_size(path)
    
    for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
        if size < 1024:
            return f"{size:.2f} {unit}"
        size /= 1024
    
    return f"{size:.2f} PB"


# ==================== 数据验证工具 ====================

def is_valid_email(email: str) -> bool:
    """验证邮箱格式"""
    pattern = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
    return bool(re.match(pattern, email))


def is_valid_phone(phone: str) -> bool:
    """验证手机号格式"""
    pattern = r'^1[3-9]\d{9}$'
    return bool(re.match(pattern, phone))


def is_valid_url(url: str) -> bool:
    """验证URL格式"""
    pattern = r'^https?://[^\s<>"{}|\\^`\[\]]+$'
    return bool(re.match(pattern, url))


def is_valid_id_card(id_card: str) -> bool:
    """验证身份证号格式（简单验证）"""
    pattern = r'^\d{17}[\dXx]$'
    return bool(re.match(pattern, id_card))


def is_json(text: str) -> bool:
    """验证是否为JSON"""
    try:
        json.loads(text)
        return True
    except:
        return False


# ==================== 数据转换工具 ====================

def deep_get(data: Dict, path: str, default: Any = None) -> Any:
    """
    深度获取字典值
    
    Args:
        data: 字典数据
        path: 路径，如 "user.profile.name"
        default: 默认值
        
    Returns:
        Any: 值
    """
    keys = path.split('.')
    result = data
    
    for key in keys:
        if isinstance(result, dict) and key in result:
            result = result[key]
        else:
            return default
    
    return result


def deep_set(data: Dict, path: str, value: Any):
    """
    深度设置字典值
    
    Args:
        data: 字典数据
        path: 路径
        value: 值
    """
    keys = path.split('.')
    current = data
    
    for key in keys[:-1]:
        if key not in current:
            current[key] = {}
        current = current[key]
    
    current[keys[-1]] = value


def flatten_dict(data: Dict, separator: str = '.') -> Dict:
    """展平嵌套字典"""
    result = {}
    
    def _flatten(obj, prefix=''):
        if isinstance(obj, dict):
            for key, value in obj.items():
                new_key = f"{prefix}{separator}{key}" if prefix else key
                _flatten(value, new_key)
        else:
            result[prefix] = obj
    
    _flatten(data)
    return result


def unflatten_dict(data: Dict, separator: str = '.') -> Dict:
    """还原展平的字典"""
    result = {}
    
    for key, value in data.items():
        deep_set(result, key, value)
    
    return result


def list_to_dict(lst: List, key_field: str) -> Dict:
    """
    列表转字典
    
    Args:
        lst: 列表
        key_field: 作为键的字段
        
    Returns:
        Dict: 字典
    """
    return {item[key_field]: item for item in lst}


def group_by(lst: List, key_field: str) -> Dict[str, List]:
    """
    按字段分组
    
    Args:
        lst: 列表
        key_field: 分组字段
        
    Returns:
        Dict: 分组结果
    """
    result = {}
    
    for item in lst:
        key = item.get(key_field)
        if key not in result:
            result[key] = []
        result[key].append(item)
    
    return result


# ==================== 重试和等待工具 ====================

async def wait_for(
    condition: callable,
    timeout: float = 30.0,
    interval: float = 0.5,
) -> bool:
    """
    等待条件满足
    
    Args:
        condition: 条件函数
        timeout: 超时时间（秒）
        interval: 检查间隔（秒）
        
    Returns:
        bool: 是否满足
    """
    import asyncio
    
    start = datetime.now()
    
    while (datetime.now() - start).total_seconds() < timeout:
        try:
            if asyncio.iscoroutinefunction(condition):
                if await condition():
                    return True
            else:
                if condition():
                    return True
        except:
            pass
        
        await asyncio.sleep(interval)
    
    return False


async def retry(
    func: callable,
    max_retries: int = 3,
    delay: float = 1.0,
    backoff: float = 2.0,
) -> Any:
    """
    重试执行
    
    Args:
        func: 要执行的函数
        max_retries: 最大重试次数
        delay: 初始延迟
        backoff: 退避因子
        
    Returns:
        Any: 执行结果
    """
    import asyncio
    
    last_error = None
    current_delay = delay
    
    for attempt in range(max_retries):
        try:
            if asyncio.iscoroutinefunction(func):
                return await func()
            else:
                return func()
        except Exception as e:
            last_error = e
            logger.warning(f"重试 {attempt + 1}/{max_retries}: {e}")
            
            if attempt < max_retries - 1:
                await asyncio.sleep(current_delay)
                current_delay *= backoff
    
    raise last_error


# ==================== 装饰器工具 ====================

def timer(func):
    """计时装饰器"""
    import time
    import functools
    
    @functools.wraps(func)
    async def async_wrapper(*args, **kwargs):
        start = time.time()
        result = await func(*args, **kwargs)
        elapsed = time.time() - start
        logger.info(f"{func.__name__} 执行耗时: {elapsed:.3f}s")
        return result
    
    @functools.wraps(func)
    def sync_wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        elapsed = time.time() - start
        logger.info(f"{func.__name__} 执行耗时: {elapsed:.3f}s")
        return result
    
    import asyncio
    if asyncio.iscoroutinefunction(func):
        return async_wrapper
    return sync_wrapper


def catch_errors(default=None):
    """错误捕获装饰器"""
    import functools
    
    def decorator(func):
        @functools.wraps(func)
        async def async_wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except Exception as e:
                logger.error(f"{func.__name__} 执行失败: {e}")
                return default
        
        @functools.wraps(func)
        def sync_wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                logger.error(f"{func.__name__} 执行失败: {e}")
                return default
        
        import asyncio
        if asyncio.iscoroutinefunction(func):
            return async_wrapper
        return sync_wrapper
    
    return decorator
